@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# Copyright 2004 Bob Friesenhahn <bfriesen@simple.dallas.tx.us>
+# Copyright 2004-2010 Bob Friesenhahn <bfriesen@simple.dallas.tx.us>
 #
 # This file is part of Magick++, the C++ API for GraphicsMagick and
 # ImageMagick.  Please see the file "COPYING" included with Magick++
@@ -14,16 +14,8 @@ cd Magick++/demo || exit 1
 
 executable=`echo $0 | sed -e 's:.*/::g;s:test_::;s:\.sh::'`
 
-outfile="test_${executable}.out"
-
-rm -f $outfile
-${RUNENV} ${MEMCHECK} ./analyze "$SRCDIR/model.miff" 2>&1 > $outfile
+${MEMCHECK} ./analyze "$SRCDIR/model.miff"
 status=$?
-if test $status -eq 1
-then
-  cat $outfile
-fi
-rm -f $outfile
 exit $status
 
 
