@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 GraphicsMagick Group
+% Copyright (C) 2003 - 2008 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -24,6 +24,8 @@
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
+%                              Jaroslav Fojtik                                %
+%                                2008 - 2009                                  %
 %                                                                             %
 %                                                                             %
 %                                                                             %
@@ -37,12 +39,13 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
-#include "magick/cache.h"
-#include "magick/color.h"
+#include "magick/colormap.h"
 #include "magick/constitute.h"
 #include "magick/magick.h"
 #include "magick/monitor.h"
+#include "magick/pixel_cache.h"
 #include "magick/utility.h"
+#include "magick/version.h"
 
 /*
   Forward declarations.
@@ -50,6 +53,26 @@
 static unsigned int
   WriteFITSImage(const ImageInfo *,Image *);
 
+
+#define FITS_BLOCK_SIZE 2880
+#define FITS_ROW_SIZE 80
+
+
+/* Convert signed values to unsigned - and reverse. */
+static void FixSignedValues(unsigned char *data, int size, unsigned step, unsigned endian)
+{
+  if(endian != MSBEndian)
+  {
+    data += step - 1;  /* LSB has most signifficant byte at the end */
+  }		       /* MSB has most signifficant byte first */
+
+  while(size-->0)
+  {
+    *data ^= 0x80;	
+    data += step;
+  }     
+}
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -127,6 +150,7 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
 {
   typedef struct _FITSInfo
   {
+    char extensions_exist;
     int
       simple,
       bits_per_pixel,
@@ -143,58 +167,31 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
   } FITSInfo;
 
   char
-    keyword[MaxTextExtent],
-    value[MaxTextExtent];
-
-  double
-    exponential[2048],
-    pixel,
-    scale,
-    scaled_pixel;
+    keyword[FITS_ROW_SIZE+1],
+    value[FITS_ROW_SIZE+1];  
 
   FITSInfo
     fits_info;
 
   Image
-    *image;
-
-  IndexPacket
-    index;
+    *image; 
 
   int
-    c,
-    quantum;
+    c;   
+
+  int logging;
 
   long
-    exponent,
-    j,
-    k,
-    l,
     packet_size,
     scene,
-    y;
-
-  register IndexPacket
-    *indexes;
-
-  register long
-    x;
+    y,
+    ax_number;
 
   register PixelPacket
     *q;
 
-  register long
-    i;
-
-  register unsigned char
-    *p;
-
-  size_t
-    count;
-
   unsigned char
-    *fits_pixels,
-    long_quantum[8];
+    *fits_pixels;    
 
   unsigned int
     status,
@@ -203,6 +200,8 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
   unsigned long
     number_pixels;
 
+  ImportPixelAreaOptions import_options;
+
   /*
     Open image file.
   */
@@ -210,31 +209,47 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
   assert(image_info->signature == MagickSignature);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
+
+  logging = LogMagickEvent(CoderEvent,GetMagickModule(),"enter"); 
+
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
     ThrowReaderException(FileOpenError,UnableToOpenFile,image);
+
   /*
-    Initialize image header.
+    Initialize common part of image header.
   */
+  fits_info.extensions_exist=0;
   fits_info.simple=False;
+
+  ImportPixelAreaOptionsInit(&import_options);
+  import_options.endian = MSBEndian;
+
+  /*
+    Decode image header.
+  */
+  if ((c=ReadBlobByte(image)) == EOF)
+    ThrowReaderException(CorruptImageError,UnexpectedEndOfFile,image);
+
+ReadExtension:
+  if (logging) (void)LogMagickEvent(CoderEvent,GetMagickModule(),
+          "Reading FITS HDU at position: %Xh", (unsigned)TellBlob(image) );
+
+  /*
+    Initialize image header for all subheaders.
+  */
   fits_info.bits_per_pixel=8;
   fits_info.columns=1;
   fits_info.rows=1;
+  fits_info.number_axes=0;
   fits_info.number_scenes=1;
   fits_info.min_data=0.0;
   fits_info.max_data=0.0;
   fits_info.zero=0.0;
   fits_info.scale=1.0;
-  /*
-    Decode image header.
-  */
-  c=ReadBlobByte(image);
-  if (c == EOF)
-    {
-      DestroyImage(image);
-      return((Image *) NULL);
-    }
+  number_pixels = 0;
+
   for ( ; ; )
   {
     if (!isalnum((int) c))
@@ -250,9 +265,10 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
         p=keyword;
         do
         {
-          if ((p-keyword) < (MaxTextExtent-1))
+          if ((p-keyword) < (FITS_ROW_SIZE-1))
             *p++=c;
-          c=ReadBlobByte(image);
+          if ((c=ReadBlobByte(image)) == EOF)
+            ThrowReaderException(CorruptImageError,UnexpectedEndOfFile,image);
         } while (isalnum(c) || (c == '_'));
         *p='\0';
         if (LocaleCompare(keyword,"END") == 0)
@@ -262,295 +278,116 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
         {
           if (c == '=')
             value_expected=True;
-          c=ReadBlobByte(image);
+          if ((c=ReadBlobByte(image)) == EOF)
+            ThrowReaderException(CorruptImageError,UnexpectedEndOfFile,image);
         }
         if (value_expected == False)
           continue;
         p=value;
+        if (c == '\'')
+          c=ReadBlobByte(image);
+
         while (isalnum(c) || (c == '-') || (c == '+') || (c == '.'))
         {
-          if ((p-value) < (MaxTextExtent-1))
+          if ((p-value) < (FITS_ROW_SIZE-1))
             *p++=c;
-          c=ReadBlobByte(image);
+          if ((c=ReadBlobByte(image)) == EOF)
+            ThrowReaderException(CorruptImageError,UnexpectedEndOfFile,image);
         }
         *p='\0';
         /*
           Assign a value to the specified keyword.
         */
         if (LocaleCompare(keyword,"SIMPLE") == 0)
-          fits_info.simple=(*value == 'T') || (*value == 't');
+          fits_info.simple = (*value=='T') || (*value=='t');
+        if (LocaleCompare(keyword,"EXTEND") == 0)
+          fits_info.extensions_exist = (*value== 'T') || (*value=='t');
         if (LocaleCompare(keyword,"BITPIX") == 0)
-          fits_info.bits_per_pixel=atoi(value);
-        if (LocaleCompare(keyword,"NAXIS") == 0)
-          fits_info.number_axes=atoi(value);
-        if (LocaleCompare(keyword,"NAXIS1") == 0)
-          fits_info.columns= atoi(value);
-        if (LocaleCompare(keyword,"NAXIS2") == 0)
-          fits_info.rows= atoi(value);
-        if (LocaleCompare(keyword,"NAXIS3") == 0)
-          fits_info.number_scenes=atoi(value);
+        {
+          fits_info.bits_per_pixel=MagickAtoI(value);
+	  if(fits_info.bits_per_pixel>0)
+            import_options.sample_type = UnsignedQuantumSampleType;
+	  if(fits_info.bits_per_pixel<0)          
+            import_options.sample_type = FloatQuantumSampleType;
+        }
+	if(!LocaleNCompare(keyword,"NAXIS",5))
+        {
+          if (keyword[5] == 0) ax_number=-1;          
+          else
+	  {
+            if(isdigit((int) keyword[5]))
+              ax_number = MagickAtoI(keyword+5);
+            else ax_number=-2;			/*unsupported fits keyword*/
+          }
+	  y=0;
+          if(ax_number>=-1)
+	    y = MagickAtoI(value);
+          switch(ax_number)
+          {
+            case -1:fits_info.number_axes = y; break;
+            case 1: fits_info.columns = (y<=0) ? 1 : y; break;
+	    case 2: fits_info.rows = (y<=0) ? 1 : y; break;
+	    case 3: fits_info.number_scenes = (y<=0) ? 1 : y; break;
+          }
+          if(ax_number>0)
+          {
+            if(number_pixels == 0)
+              number_pixels = y;
+            else
+              number_pixels *= (y<=0) ? 1 : y;
+          }
+        }
+
         if (LocaleCompare(keyword,"DATAMAX") == 0)
-          fits_info.max_data=atof(value);
+          fits_info.max_data=MagickAtoF(value);
         if (LocaleCompare(keyword,"DATAMIN") == 0)
-          fits_info.min_data=atof(value);
+          fits_info.min_data=MagickAtoF(value);
         if (LocaleCompare(keyword,"BZERO") == 0)
-          fits_info.zero=atof(value);
+          fits_info.zero=MagickAtoF(value);
         if (LocaleCompare(keyword,"BSCALE") == 0)
-          fits_info.scale=atof(value);
+          fits_info.scale=MagickAtoF(value);
+        if (LocaleCompare(keyword,"XENDIAN") == 0)
+        {
+	  if (LocaleCompare(keyword,"BIG") == 0)
+	    import_options.endian = MSBEndian;
+          else
+	    import_options.endian = LSBEndian;          
+	}
       }
     while ((TellBlob(image) % 80) != 0)
-      c=ReadBlobByte(image);
-    c=ReadBlobByte(image);
+      if ((c=ReadBlobByte(image)) == EOF)
+        ThrowReaderException(CorruptImageError,UnexpectedEndOfFile,image);
+    if ((c=ReadBlobByte(image)) == EOF)
+      ThrowReaderException(CorruptImageError,UnexpectedEndOfFile,image);
   }
-  while ((TellBlob(image) % 2880) != 0)
-    c=ReadBlobByte(image);
+
+  while ((TellBlob(image) % FITS_BLOCK_SIZE) != 0)		/* Read till the rest of a block. */
+    if ((c=ReadBlobByte(image)) == EOF)
+      ThrowReaderException(CorruptImageError,UnexpectedEndOfFile,image);
+
   /*
     Verify that required image information is defined.
-  */
-  number_pixels=fits_info.columns*fits_info.rows;
+  */  
+  if (logging) (void)LogMagickEvent(CoderEvent,GetMagickModule(),
+          "HDU read finished at %Xh, number of pixel is: %d (%d*%d*%d)", (unsigned)TellBlob(image), (unsigned)number_pixels,
+          (unsigned)fits_info.columns, (unsigned)fits_info.rows, (unsigned)fits_info.number_scenes);
+
   if ((!fits_info.simple) || (fits_info.number_axes < 1) ||
       (fits_info.number_axes > 4) || (number_pixels == 0))
-    ThrowReaderException(CorruptImageError,ImageTypeNotSupported,image);
-  if (fits_info.bits_per_pixel == -32)
-    {
-      exponential[150]=1.0;
-      for (i=151; i < 256; i++)
-        exponential[i]=2.0*exponential[i-1];
-      for (i=149; i >= 0; i--)
-        exponential[i]=exponential[i+1]/2.0;
-    }
-  if (fits_info.bits_per_pixel == -64)
-    {
-      exponential[1075]=1.0;
-      for (i=1076; i < 2048; i++)
-        exponential[i]=2.0*exponential[i-1];
-      for (i=1074; i >= 0; i--)
-        exponential[i]=exponential[i+1]/2.0;
-    }
-  for (scene=0; scene < (long) fits_info.number_scenes; scene++)
   {
-    /*
-      Create linear colormap.
-    */
-    image->columns=fits_info.columns;
-    image->rows=fits_info.rows;
-    image->depth=fits_info.bits_per_pixel <= 8 ? 8 : QuantumDepth;
-    image->storage_class=PseudoClass;
-    image->scene=scene;
-    if (!AllocateImageColormap(image,1 << image->depth))
-      ThrowReaderException(FileOpenError,UnableToOpenFile,image);
-    if (image_info->ping && (image_info->subrange != 0))
-      if (image->scene >= (image_info->subimage+image_info->subrange-1))
-        break;
-    /*
-      Initialize image structure.
-    */
-    packet_size=fits_info.bits_per_pixel/8;
-    if (packet_size < 0)
-      packet_size=(-packet_size);
-    number_pixels=image->columns*image->rows;
-    fits_pixels=MagickAllocateMemory(unsigned char *,packet_size*number_pixels);
-    if (fits_pixels == (unsigned char *) NULL)
-      ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
-    /*
-      Convert FITS pixels to pixel packets.
-    */
-    count=ReadBlob(image,packet_size*number_pixels,fits_pixels);
-    if (count == 0)
-      ThrowReaderException(CorruptImageError,InsufficientImageDataInFile,
-        image);
-    if ((fits_info.min_data == 0.0) && (fits_info.max_data == 0.0))
-      {
-        /*
-          Determine minimum and maximum intensity.
-        */
-        p=fits_pixels;
-        long_quantum[0]=(*p);
-        quantum=(*p++);
-        for (j=1; j <= (packet_size-1); j++)
-        {
-          long_quantum[j]=(*p);
-          quantum=(quantum << 8) | (*p++);
-        }
-        pixel=(double) quantum;
-        if (fits_info.bits_per_pixel == 16)
-          if (pixel > 32767)
-            pixel-=65536L;
-        if (fits_info.bits_per_pixel == -32)
-          {
-            j=((long) long_quantum[1] << 16) | ((long) long_quantum[2] << 8) |
-               (long) long_quantum[3];
-            k=(int) *long_quantum;
-            exponent=((k & 127) << 1) | (j >> 23);
-            *(float *) long_quantum=
-              exponential[exponent]*(float) (j | 0x800000);
-            if ((exponent | j) == 0)
-              *(float *) long_quantum=0.0;
-            if (k & 128)
-              *(float *) long_quantum=(-(*(float *) long_quantum));
-            pixel=(double) (*((float *) long_quantum));
-          }
-        if (fits_info.bits_per_pixel == -64)
-          {
-            j=((long) long_quantum[1] << 24) |
-              ((long) long_quantum[2] << 16) |
-              ((long) long_quantum[3] << 8) |
-               (long) long_quantum[4];
-            k=(int) *long_quantum;
-            l=((int) long_quantum[5] << 16) | ((int) long_quantum[6] << 8) |
-               (int) long_quantum[7];
-            exponent=((k & 127) << 4) | (j >> 28);
-            *(double *) long_quantum=exponential[exponent]*(16777216.0*
-              (double) ((j & 0x0FFFFFFF) | 0x10000000)+(double) l);
-            if ((exponent | j | l) == 0)
-              *(double *) long_quantum=0.0;
-            if (k & 128)
-              *(double *)long_quantum=(-(*(double *) long_quantum));
-            pixel=(double) (*((double *) long_quantum));
-          }
-        fits_info.min_data=pixel*fits_info.scale-fits_info.zero;
-        fits_info.max_data=pixel*fits_info.scale-fits_info.zero;
-        for (i=1; i < (long) number_pixels; i++)
-        {
-          long_quantum[0]=(*p);
-          quantum=(*p++);
-          for (j=1; j <= (packet_size-1); j++)
-          {
-            long_quantum[j]=(*p);
-            quantum=(quantum << 8) | (*p++);
-          }
-          pixel=(double) quantum;
-          if (fits_info.bits_per_pixel == 16)
-            if (pixel > 32767)
-              pixel-=65536L;
-          if (fits_info.bits_per_pixel == -32)
-            {
-              j=((long) long_quantum[1] << 16) | ((long) long_quantum[2] << 8) |
-                 (long) long_quantum[3];
-              k=(int) *long_quantum;
-              exponent=((k & 127) << 1) | (j >> 23);
-              *(float *) long_quantum=
-                exponential[exponent]*(float) (j | 0x800000);
-              if ((exponent | j) == 0)
-                *(float *) long_quantum=0.0;
-              if (k & 128)
-                *(float *) long_quantum=(-(*(float *) long_quantum));
-              pixel=(double) (*((float *) long_quantum));
-            }
-          if (fits_info.bits_per_pixel == -64)
-            {
-              j=((long) long_quantum[1] << 24) |
-                ((long) long_quantum[2] << 16) |
-                ((long) long_quantum[3] << 8) |
-                 (long) long_quantum[4];
-              k=(int) *long_quantum;
-              l=((int) long_quantum[5] << 16) | ((int) long_quantum[6] << 8) |
-                 (int) long_quantum[7];
-              exponent=((k & 127) << 4) | (j >> 28);
-              *(double *) long_quantum=exponential[exponent]*(16777216.0*
-                (double) ((j & 0x0FFFFFFF) | 0x10000000)+(double) l);
-              if ((exponent | j | l) == 0)
-                *(double *) long_quantum=0.0;
-              if (k & 128)
-                *(double *)long_quantum=(-(*(double *) long_quantum));
-              pixel=(double) (*((double *) long_quantum));
-            }
-          scaled_pixel=pixel*fits_info.scale-fits_info.zero;
-          if (scaled_pixel < fits_info.min_data)
-            fits_info.min_data=scaled_pixel;
-          if (scaled_pixel > fits_info.max_data)
-            fits_info.max_data=scaled_pixel;
-        }
-      }
-    /*
-      Convert FITS pixels to pixel packets.
-    */
-    scale=1.0;
-    if ((fits_info.bits_per_pixel < 0) ||
-        ((fits_info.max_data-fits_info.min_data) > ((1 << image->depth)-1)))
-      scale=((1 << image->depth)-1)/(fits_info.max_data-fits_info.min_data);
-    p=fits_pixels;
-    for (y=(long) image->rows-1; y >= 0; y--)
-    {
-      q=SetImagePixels(image,0,y,image->columns,1);
-      if (q == (PixelPacket *) NULL)
-        break;
-      indexes=GetIndexes(image);
-      for (x=0; x < (long) image->columns; x++)
-      {
-        long_quantum[0]=(*p);
-        quantum=(*p++);
-        for (j=1; j <= (packet_size-1); j++)
-        {
-          long_quantum[j]=(*p);
-          quantum=(quantum << 8) | (*p++);
-        }
-        pixel=(double) quantum;
-        if (fits_info.bits_per_pixel == 16)
-          if (pixel > 32767)
-            pixel-=65536L;
-        if (fits_info.bits_per_pixel == -32)
-          {
-            j=((long) long_quantum[1] << 16) | ((long) long_quantum[2] << 8) |
-               (long) long_quantum[3];
-            k=(int) *long_quantum;
-            exponent=((k & 127) << 1) | (j >> 23);
-            *(float *) long_quantum=
-              exponential[exponent]*(float) (j | 0x800000);
-            if ((exponent | j) == 0)
-              *(float *) long_quantum=0.0;
-            if (k & 128)
-              *(float *) long_quantum=(-(*(float *) long_quantum));
-            pixel=(double) (*((float *) long_quantum));
-          }
-        if (fits_info.bits_per_pixel == -64)
-          {
-            j=((long) long_quantum[1] << 24) |
-              ((long) long_quantum[2] << 16) |
-              ((long) long_quantum[3] << 8) |
-               (long) long_quantum[4];
-            k=(int) *long_quantum;
-            l=((long) long_quantum[5] << 16) | ((long) long_quantum[6] << 8) |
-               (long) long_quantum[7];
-            exponent=((k & 127) << 4) | (j >> 28);
-            *(double *) long_quantum=exponential[exponent]*(16777216.0*
-              (double) ((j & 0x0FFFFFFF) | 0x10000000)+(double) l);
-            if ((exponent | j | l) == 0)
-              *(double *) long_quantum=0.0;
-            if (k & 128)
-              *(double *)long_quantum=(-(*(double *) long_quantum));
-            pixel=(double) (*((double *) long_quantum));
-          }
-        scaled_pixel=scale*
-          (pixel*fits_info.scale-fits_info.min_data-fits_info.zero);
-        index=(IndexPacket) ((scaled_pixel < 0) ? 0 :
-          (scaled_pixel > ((1 << image->depth)-1)) ? ((1 << image->depth)-1) :
-          scaled_pixel+0.5);
-        index=(IndexPacket) (index);
-        VerifyColormapIndex(image,index);
-        indexes[x]=index;
-        *q++=image->colormap[index];
-      }
-      if (!SyncImagePixels(image))
-        break;
-      if (QuantumTick(y,image->rows))
-        if (!MagickMonitor(LoadImageText,y,image->rows,exception))
-          break;
-    }
-    MagickFreeMemory(fits_pixels);
-    if (EOFBlob(image))
-      {
-        ThrowException(exception,CorruptImageError,UnexpectedEndOfFile,
-          image->filename);
-        break;
-      }
-    /*
-      Proceed to next image.
-    */
-    if (image_info->subrange != 0)
-      if (image->scene >= (image_info->subimage+image_info->subrange-1))
-        break;
-    if (scene < (long) (fits_info.number_scenes-1))
+    if(!fits_info.extensions_exist)	/* when extensions exists, process further */
+      ThrowReaderException(CorruptImageError,ImageTypeNotSupported,image);
+
+    number_pixels = (number_pixels*fits_info.bits_per_pixel) / 8;
+    number_pixels = ((number_pixels+FITS_BLOCK_SIZE-1) / FITS_BLOCK_SIZE) * FITS_BLOCK_SIZE; /* raw data block size */
+    (void) SeekBlob(image,number_pixels,SEEK_CUR);    
+  }
+  else
+  {
+   number_pixels = fits_info.columns*fits_info.rows; 
+   for (scene=0; scene < (long) fits_info.number_scenes; scene++)
+   {
+      if(image->rows!=0 && image->columns!=0)
       {
         /*
           Allocate next image structure.
@@ -562,13 +399,147 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        if (!MagickMonitor(LoadImagesText,TellBlob(image),GetBlobSize(image),exception))
+        if (!MagickMonitorFormatted(TellBlob(image),GetBlobSize(image),exception,
+                                    LoadImagesText,image->filename))
           break;
       }
+
+    /*
+      Create linear colormap.
+    */
+    image->columns = fits_info.columns;
+    image->rows = fits_info.rows;
+    if(fits_info.bits_per_pixel>=0)
+      image->depth = Min(QuantumDepth,fits_info.bits_per_pixel);
+    else
+      image->depth = QuantumDepth;		/* double type cell */
+    /* image->storage_class=PseudoClass; */
+    image->storage_class = DirectClass;
+    image->scene=scene;
+    image->is_grayscale = 1;
+
+    if (image->depth<=8 && fits_info.bits_per_pixel==8)
+      if (!AllocateImageColormap(image,1 << image->depth))
+        ThrowReaderException(FileOpenError,UnableToOpenFile,image);
+
+    if (image_info->ping && (image_info->subrange != 0))
+      if (image->scene >= (image_info->subimage+image_info->subrange-1))
+        break;
+
+    /*
+      Initialize image structure.
+    */
+    packet_size=fits_info.bits_per_pixel/8;
+    if (packet_size < 0) packet_size = -packet_size;
+        
+    number_pixels = image->columns*image->rows;
+    if ((number_pixels / image->columns) != image->rows)
+      ThrowReaderException(CoderError,ImageColumnOrRowSizeIsNotSupported,image);
+
+    fits_pixels=MagickAllocateArray(unsigned char *, image->columns, packet_size);
+    if (fits_pixels == (unsigned char *) NULL)
+      ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+
+    /*
+      Recover minimum and maximum from data if not present in HDU
+    */     
+    if ((fits_info.min_data == 0.0) && (fits_info.max_data == 0.0))
+    {  /*  Determine minimum and maximum intensity. */
+      if(fits_info.bits_per_pixel==-32)
+        (void) MagickFindRawImageMinMax(image, import_options.endian, image->columns,
+                                        image->rows, FloatPixel, packet_size*image->columns,
+                                        fits_pixels, &import_options.double_minvalue,
+                                        &import_options.double_maxvalue);
+      if(fits_info.bits_per_pixel==-64)
+        (void) MagickFindRawImageMinMax(image, import_options.endian, image->columns,
+                                        image->rows, DoublePixel, packet_size*image->columns,
+                                        fits_pixels, &import_options.double_minvalue,
+                                        &import_options.double_maxvalue);
+    }
+    else
+    {
+      import_options.double_maxvalue = fits_info.max_data;
+      import_options.double_minvalue = fits_info.min_data;
+    }
+
+    /*
+      Convert FITS pixels to pixel packets.
+    */
+
+    for (y=(long) image->rows-1; y >= 0; y--)
+    {
+      q=SetImagePixels(image,0,y,image->columns,1);
+      if (q == (PixelPacket *) NULL)
+        break;
+
+      if(ReadBlob(image, packet_size*image->columns, fits_pixels) != (size_t)packet_size*image->columns)
+      {
+ 	if (logging) (void)LogMagickEvent(CoderEvent,GetMagickModule(),
+           "  fits cannot read scanrow %u from a file.", (unsigned)y );
+	break; /* goto ExitLoop; */
+      }
+
+      switch(fits_info.bits_per_pixel)
+      {
+        case 16: FixSignedValues(fits_pixels, image->columns, 2, import_options.endian);
+		 break;
+        case 32: FixSignedValues(fits_pixels, image->columns, 4, import_options.endian);
+		 break;
+	case 64: FixSignedValues(fits_pixels, image->columns, 8, import_options.endian);
+		 break;
+      }
+
+      if(ImportImagePixelArea(image, GrayQuantum, packet_size*8, fits_pixels, &import_options,0) == MagickFail)
+      {
+        if (logging) (void)LogMagickEvent(CoderEvent,GetMagickModule(),
+               "  fits failed to ImportImagePixelArea for a row %u", (unsigned)y);
+	break;
+      }
+
+      if (!SyncImagePixels(image))
+        break;
+      if (QuantumTick(y,image->rows))
+        if (!MagickMonitorFormatted(y,image->rows,exception,
+                                    LoadImageText,image->filename,
+				    image->columns,image->rows))
+          break;
+    }
+    MagickFreeMemory(fits_pixels);
+    if (EOFBlob(image))
+      {
+        ThrowException(exception,CorruptImageError,UnexpectedEndOfFile, image->filename);
+        break;
+      }
+
+    /*
+      Proceed to next image.
+    */
+    if (image_info->subrange != 0)
+      if (image->scene >= (image_info->subimage+image_info->subrange-1))
+        break;    
+    }
   }
-  while (image->previous != (Image *) NULL)
-    image=image->previous;
+
+  if(fits_info.extensions_exist)
+  {
+    while ((TellBlob(image) % FITS_BLOCK_SIZE) != 0) /* Read till the rest of a block. */
+      if ((c=ReadBlobByte(image)) == EOF)
+        break;
+    if(!EOFBlob(image))
+    {						/* Try to read a next extension block header. */
+      if ((c=ReadBlobByte(image)) != EOF)
+	goto ReadExtension;
+    }
+  }
+
   CloseBlob(image);
+
+  while (image->previous != (Image *) NULL)
+    image=image->previous;  
+
+  if (logging) (void)LogMagickEvent(CoderEvent,GetMagickModule(),"return");  
+  if(image==NULL)
+    ThrowReaderException(CorruptImageError,ImageFileDoesNotContainAnyImageData,image);
   return(image);
 }
 
@@ -605,8 +576,8 @@ ModuleExport void RegisterFITSImage(void)
   entry->encoder=(EncoderHandler) WriteFITSImage;
   entry->magick=(MagickHandler) IsFITS;
   entry->adjoin=False;
-  entry->description=AcquireString("Flexible Image Transport System");
-  entry->module=AcquireString("FITS");
+  entry->description="Flexible Image Transport System";
+  entry->module="FITS";
   (void) RegisterMagickInfo(entry);
 }
 
@@ -634,6 +605,31 @@ ModuleExport void UnregisterFITSImage(void)
   (void) UnregisterMagickInfo("FITS");
 }
 
+
+/*
+  This functions inserts one row into a HDU. Note that according to
+  FITS spec a card image contains 80 bytes of ASCII data.
+  
+  buffer - 2880 byte logical FITS record.
+  data   - string data to append
+  offset - offset into FITS record to write the data.
+*/
+int InsertRowHDU(char *buffer, const char *data, int offset)
+{
+  int
+    len;
+
+  if (data == NULL)
+    return offset;
+  len = strlen(data);
+  len = Min(len,80); /* Each card image is 80 bytes max */
+
+  if (len > FITS_BLOCK_SIZE-offset) len = FITS_BLOCK_SIZE-offset;
+
+  (void) strncpy(buffer+offset,data,len);
+  return offset +80;
+}
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -667,27 +663,27 @@ ModuleExport void UnregisterFITSImage(void)
 static unsigned int WriteFITSImage(const ImageInfo *image_info,Image *image)
 {
   char
-    buffer[81],
+    buffer[FITS_BLOCK_SIZE],
     *fits_info;
 
   long
     y;
 
   register const PixelPacket
-    *p;
-
-  register long
-    i;
+    *p;  
 
   unsigned char
     *pixels;
 
-  unsigned int
+  unsigned int    
+    quantum_size,
     status;
 
   unsigned long
     packet_size;
 
+  ExportPixelAreaOptions export_options;
+
   /*
     Open output image file.
   */
@@ -698,40 +694,62 @@ static unsigned int WriteFITSImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
     ThrowWriterException(FileOpenError,UnableToOpenFile,image);
-  TransformColorspace(image,RGBColorspace);
+  (void) TransformColorspace(image,RGBColorspace);
+
+  ExportPixelAreaOptionsInit(&export_options);
+  export_options.endian = MSBEndian;
+  export_options.sample_type = UnsignedQuantumSampleType;
+
+  if (image->depth <= 8)
+  {
+    quantum_size=8;    
+  }
+  else if (image->depth <= 16)
+  {    
+    quantum_size=16;
+  }
+  else
+  {   
+    quantum_size=32;
+  }
+
   /*
     Allocate image memory.
-  */
-  packet_size=image->depth > 8 ? 2 : 1;
-  fits_info=MagickAllocateMemory(char *,2880);
+  */  
+  packet_size=quantum_size/8;
+  fits_info=MagickAllocateMemory(char *,FITS_BLOCK_SIZE);
   pixels=MagickAllocateMemory(unsigned char *,packet_size*image->columns);
   if ((fits_info == (char *) NULL) || (pixels == (unsigned char *) NULL))
     ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
+
   /*
     Initialize image header.
-  */
-  for (i=0; i < 2880; i++)
-    fits_info[i]=' ';
-  (void) strcpy(buffer,"SIMPLE  =                    T");
-  (void) strncpy(fits_info+0,buffer,strlen(buffer));
-  FormatString(buffer,"BITPIX  =                    %ld",image->depth);
-  (void) strncpy(fits_info+80,buffer,strlen(buffer));
-  (void) strcpy(buffer,"NAXIS   =                    2");
-  (void) strncpy(fits_info+160,buffer,strlen(buffer));
-  FormatString(buffer,"NAXIS1  =           %10lu",image->columns);
-  (void) strncpy(fits_info+240,buffer,strlen(buffer));
-  FormatString(buffer,"NAXIS2  =           %10lu",image->rows);
-  (void) strncpy(fits_info+320,buffer,strlen(buffer));
-  FormatString(buffer,"DATAMIN =           %10u",0);
-  (void) strncpy(fits_info+400,buffer,strlen(buffer));
-  FormatString(buffer,"DATAMAX =           %10u",(1 << image->depth)-1);
-  (void) strncpy(fits_info+480,buffer,strlen(buffer));
-  (void) strcpy(buffer,"HISTORY Created by GraphicsMagick.");
-  (void) strncpy(fits_info+560,buffer,strlen(buffer));
-  (void) strcpy(buffer,"END");
-  (void) strncpy(fits_info+640,buffer,strlen(buffer));
-  (void) WriteBlob(image,2880,(char *) fits_info);
-  MagickFreeMemory(fits_info);
+  */  
+  memset(fits_info,' ',FITS_BLOCK_SIZE);
+  y = 0;
+  y = InsertRowHDU(fits_info, "SIMPLE  =                    T", y);
+  FormatString(buffer,        "BITPIX  =                    %u",quantum_size);
+  y = InsertRowHDU(fits_info, buffer, y);
+  y = InsertRowHDU(fits_info, "NAXIS   =                    2", y);
+  FormatString(buffer,        "NAXIS1  =           %10lu",image->columns);
+  y = InsertRowHDU(fits_info, buffer, y);
+  FormatString(buffer,        "NAXIS2  =           %10lu",image->rows); 
+  y = InsertRowHDU(fits_info, buffer, y);  
+  FormatString(buffer,        "DATAMIN =           %10u",0);
+  y = InsertRowHDU(fits_info, buffer, y);    
+  FormatString(buffer,        "DATAMAX =           %10lu", MaxValueGivenBits(quantum_size));
+  y = InsertRowHDU(fits_info, buffer, y);
+  if(quantum_size>8)
+  {
+    FormatString(buffer,      "BZERO   =           %10u", (quantum_size <= 16) ? 32768U : 2147483648U);
+    y = InsertRowHDU(fits_info, buffer, y);
+  }
+  /* Magick version data can only be 60 bytes. */
+  (void) FormatString(buffer, "HISTORY Created by %.60s.",MagickPackageName " " MagickLibVersionText);
+  y = InsertRowHDU(fits_info, buffer, y);
+  y = InsertRowHDU(fits_info, "END", y);        
+  (void) WriteBlob(image, FITS_BLOCK_SIZE, (char *)fits_info);
+  
   /*
     Convert image to fits scale PseudoColor class.
   */
@@ -740,16 +758,30 @@ static unsigned int WriteFITSImage(const ImageInfo *image_info,Image *image)
     p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    (void) PopImagePixels(image,GrayQuantum,pixels);
+    (void) ExportImagePixelArea(image,GrayQuantum,quantum_size,pixels,&export_options,0);
+    if(quantum_size==16) FixSignedValues(pixels, image->columns, 2, export_options.endian);
+    if(quantum_size==32) FixSignedValues(pixels, image->columns, 4, export_options.endian);
     (void) WriteBlob(image,packet_size*image->columns,pixels);
     if (QuantumTick(image->rows-y-1,image->rows))
       {
-        status=MagickMonitor(SaveImageText,image->rows-y-1,image->rows,
-          &image->exception);
+        status=MagickMonitorFormatted(image->rows-y-1,image->rows,
+                                      &image->exception,SaveImageText,
+                                      image->filename,
+				      image->columns,image->rows);
         if (status == False)
           break;
       }
   }
+  
+	/* Calculate of padding */
+  y = FITS_BLOCK_SIZE - (image->columns * image->rows * packet_size) % FITS_BLOCK_SIZE;
+  if(y>0)
+  {
+    memset(fits_info, 0, y);    
+    (void)WriteBlob(image,y,(char *) fits_info);
+  }
+  MagickFreeMemory(fits_info);
+
   MagickFreeMemory(pixels);
   CloseBlob(image);
   return(True);
