@@ -37,11 +37,13 @@
 */
 #include "magick/studio.h"
 #include "magick/attribute.h"
-#include "magick/pixel_cache.h"
 #include "magick/blob.h"
 #include "magick/color.h"
+#include "magick/color_lookup.h"
+#include "magick/colormap.h"
 #include "magick/magick.h"
 #include "magick/monitor.h"
+#include "magick/pixel_cache.h"
 #include "magick/profile.h"
 #include "magick/utility.h"
 
@@ -62,7 +64,7 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsMPC returns True if the image format type, identified by the
+%  Method IsMPC returns MagickTrue if the image format type, identified by the
 %  magick string, is an Magick Persistent Cache image.
 %
 %  The format of the IsMPC method is:
@@ -71,7 +73,7 @@ static unsigned int
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsMPC returns True if the image format type is MPC.
+%    o status:  Method IsMPC returns MagickTrue if the image format type is MPC.
 %
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
@@ -80,13 +82,13 @@ static unsigned int
 %
 %
 */
-static unsigned int IsMPC(const unsigned char *magick,const size_t length)
+static MagickBool IsMPC(const unsigned char *magick,const size_t length)
 {
   if (length < 14)
-    return(False);
+    return(MagickFalse);
   if (LocaleNCompare((char *) magick,"id=MagickCache",14) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -168,8 +170,18 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
+  /*
+    We must unset the grayscale and monochrome flags by default since
+    the MPC format does not necessarily update the pixel cache while
+    it is read.
+  */
+  image->is_grayscale=MagickFalse;
+  image->is_monochrome=MagickFalse;
+  /*
+    Open blob
+  */
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
+  if (status == MagickFail)
     ThrowReaderException(FileOpenError,UnableToOpenFile,image);
   (void) strlcpy(cache_filename,image->filename,MaxTextExtent);
   AppendImageFormat("cache",cache_filename);
@@ -314,7 +326,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   }
                 if (LocaleCompare(keyword,"colors") == 0)
                   {
-                    image->colors=atol(values);
+                    image->colors=MagickAtoL(values);
                     break;
                   }
                 if (LocaleCompare(keyword,"colorspace") == 0)
@@ -356,7 +368,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   }
                 if (LocaleCompare(keyword,"columns") == 0)
                   {
-                    image->columns= atol(values);
+                    image->columns= MagickAtoL(values);
                     break;
                   }
                 (void) SetImageAttribute(image,keyword,
@@ -368,17 +380,17 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 if (LocaleCompare(keyword,"delay") == 0)
                   {
-                    image->delay=atol(values);
+                    image->delay=MagickAtoL(values);
                     break;
                   }
                 if (LocaleCompare(keyword,"depth") == 0)
                   {
-                    image->depth=atol(values);
+                    image->depth=MagickAtoL(values);
                     break;
                   }
                 if (LocaleCompare(keyword,"dispose") == 0)
                   {
-                    image->dispose=(DisposeType) atol(values);
+                    image->dispose=(DisposeType) MagickAtoL(values);
                     if (LocaleCompare(values,"Background") == 0)
                       image->dispose=BackgroundDispose;
                     else
@@ -398,7 +410,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 if (LocaleCompare(keyword,"error") == 0)
                   {
-                    image->error.mean_error_per_pixel=atof(values);
+                    image->error.mean_error_per_pixel=MagickAtoF(values);
                     break;
                   }
                 (void) SetImageAttribute(image,keyword,
@@ -410,13 +422,13 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 if (LocaleCompare(keyword,"gamma") == 0)
                   {
-                    image->gamma=atof(values);
+                    image->gamma=MagickAtoF(values);
                     break;
                   }
                 if (LocaleCompare(keyword,"grayscale") == 0)
                   {
-                    image->is_grayscale=(LocaleCompare(values,"True") == 0) ||
-                      (LocaleCompare(values,"true") == 0);
+		    if (LocaleCompare(values,"True") == 0)
+		      image->is_grayscale=MagickTrue;
                     break;
                   }
                 if (LocaleCompare(keyword,"green-primary") == 0)
@@ -440,7 +452,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   }
                 if (LocaleCompare(keyword,"iterations") == 0)
                   {
-                    image->iterations=atol(values);
+                    image->iterations=MagickAtoL(values);
                     break;
                   }
                 (void) SetImageAttribute(image,keyword,
@@ -464,18 +476,18 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   }
                 if (LocaleCompare(keyword,"maximum-error") == 0)
                   {
-                    image->error.normalized_maximum_error=atof(values);
+                    image->error.normalized_maximum_error=MagickAtoF(values);
                     break;
                   }
                 if (LocaleCompare(keyword,"mean-error") == 0)
                   {
-                    image->error.normalized_mean_error=atof(values);
+                    image->error.normalized_mean_error=MagickAtoF(values);
                     break;
                   }
                 if (LocaleCompare(keyword,"monochrome") == 0)
                   {
-                    image->is_monochrome=(LocaleCompare(values,"True") == 0) ||
-                      (LocaleCompare(values,"true") == 0);
+		    if (LocaleCompare(values,"True") == 0)
+		      image->is_monochrome=MagickTrue;
                     break;
                   }
                 if (LocaleCompare(keyword,"montage") == 0)
@@ -508,7 +520,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     if (profiles == (ProfileInfo *) NULL)
                       ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
                     profiles[i].name=AllocateString(keyword+8);
-                    profiles[i].length=atol(values);
+                    profiles[i].length=MagickAtoL(values);
                     profiles[i].info=(unsigned char *) NULL;
                     number_of_profiles++;
                     break;
@@ -535,7 +547,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 if (LocaleCompare(keyword,"quantum-depth") == 0)
                   {
-                    quantum_depth=atol(values);
+                    quantum_depth=MagickAtoL(values);
                     break;
                   }
                 (void) SetImageAttribute(image,keyword,
@@ -576,7 +588,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   }
                 if (LocaleCompare(keyword,"rows") == 0)
                   {
-                    image->rows=atol(values);
+                    image->rows=MagickAtoL(values);
                     break;
                   }
                 (void) SetImageAttribute(image,keyword,
@@ -588,7 +600,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 if (LocaleCompare(keyword,"scene") == 0)
                   {
-                    image->scene=atol(values);
+                    image->scene=MagickAtoL(values);
                     break;
                   }
                 (void) SetImageAttribute(image,keyword,
@@ -772,8 +784,8 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
     /*
       Attach persistent pixel cache.
     */
-    status=PersistCache(image,cache_filename,True,&offset,exception);
-    if (status == False)
+    status=PersistCache(image,cache_filename,MagickTrue,&offset,exception);
+    if (status == MagickFail)
       ThrowReaderException(CacheError,UnableToPeristPixelCache,image);
     /*
       Proceed to next image.
@@ -797,7 +809,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
         status=MagickMonitorFormatted(TellBlob(image),GetBlobSize(image),
                                       exception,LoadImagesText,
                                       image->filename);
-        if (status == False)
+        if (status == MagickFail)
           break;
       }
   } while (c != EOF);
@@ -891,12 +903,12 @@ ModuleExport void UnregisterMPCImage(void)
 %
 %  The format of the WriteMPCImage method is:
 %
-%      unsigned int WriteMPCImage(const ImageInfo *image_info,Image *image)
+%      MagickPassFail WriteMPCImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows:
 %
-%    o status: Method WriteMPCImage return True if the image is written.
-%      False is returned if there is a memory shortage or if the image file
+%    o status: Method WriteMPCImage return MagickPass if the image is written.
+%      MagickFail is returned if there is a memory shortage or if the image file
 %      fails to write.
 %
 %    o image_info: Specifies a pointer to a ImageInfo structure.
@@ -905,7 +917,7 @@ ModuleExport void UnregisterMPCImage(void)
 %
 %
 */
-static unsigned int WriteMPCImage(const ImageInfo *image_info,Image *image)
+static MagickPassFail WriteMPCImage(const ImageInfo *image_info,Image *image)
 {
   char
     buffer[MaxTextExtent],
@@ -946,7 +958,7 @@ static unsigned int WriteMPCImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
+  if (status == MagickFail)
     ThrowWriterException(FileOpenError,UnableToOpenFile,image);
   (void) strlcpy(cache_filename,image->filename,MaxTextExtent);
   AppendImageFormat("cache",cache_filename);
@@ -1225,7 +1237,7 @@ static unsigned int WriteMPCImage(const ImageInfo *image_info,Image *image)
         packet_size=image->depth > 8 ? 6 : 3;
         colormap=MagickAllocateMemory(unsigned char *,packet_size*image->colors);
         if (colormap == (unsigned char *) NULL)
-          return(False);
+          return(MagickFail);
         /*
           Write colormap to file.
         */
@@ -1256,8 +1268,8 @@ static unsigned int WriteMPCImage(const ImageInfo *image_info,Image *image)
     /*
       Initialize persistent pixel cache.
     */
-    status=PersistCache(image,cache_filename,False,&offset,&image->exception);
-    if (status == False)
+    status=PersistCache(image,cache_filename,MagickFalse,&offset,&image->exception);
+    if (status == MagickFail)
       ThrowWriterException(CacheError,UnableToPeristPixelCache,image);
     if (image->next == (Image *) NULL)
       break;
