@@ -36,6 +36,7 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/analyze.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/pixel_cache.h"
@@ -137,7 +138,6 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     density[MaxTextExtent],
     filename[MaxTextExtent],
     geometry[MaxTextExtent],
-    options[MaxTextExtent],
     postscript_filename[MaxTextExtent],
     translate_geometry[MaxTextExtent];
 
@@ -158,9 +158,6 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *image,
     *next_image;
 
-  ImageInfo
-    *clone_info;
-
   int
     c,
     status;
@@ -308,31 +305,44 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   CloseBlob(image);
   filesize=GetBlobSize(image);
   DestroyImage(image);
+  image=(Image *) NULL;
   /*
     Use Ghostscript to convert Postscript image.
   */
-  *options='\0';
-  if (image_info->subrange != 0)
-    FormatString(options,"-dFirstPage=%lu -dLastPage=%lu",
-      image_info->subimage+1,image_info->subimage+image_info->subrange);
-  (void) strlcpy(filename,image_info->filename,MaxTextExtent);
-  if (image_info->temporary)
-    (void) LiberateTemporaryFile((char *) image_info->filename);
-  if(!AcquireTemporaryFileName((char *) image_info->filename))
-    {
-      (void) LiberateTemporaryFile(postscript_filename);
-      ThrowReaderTemporaryFileException(image_info->filename);
-    }
-  FormatString(command,delegate_info->commands,antialias,
-    antialias,geometry,density,options,image_info->filename,
-    postscript_filename);
+  {
+    char
+      options[MaxTextExtent];
+    
+    options[0]='\0';
+    /*
+      Append subrange.
+    */
+    if (image_info->subrange != 0)
+      FormatString(options,"-dFirstPage=%lu -dLastPage=%lu",
+		   image_info->subimage+1,image_info->subimage+image_info->subrange);
+    /*
+      Append bounding box.
+    */
+    FormatString(options+strlen(options)," -g%s",geometry);
+    (void) strlcpy(filename,image_info->filename,MaxTextExtent);
+    if (image_info->temporary)
+      (void) LiberateTemporaryFile((char *) image_info->filename);
+    if(!AcquireTemporaryFileName((char *) image_info->filename))
+      {
+	(void) LiberateTemporaryFile(postscript_filename);
+	ThrowReaderTemporaryFileException(image_info->filename);
+      }
+    FormatString(command,delegate_info->commands,antialias,
+		 antialias,density,options,image_info->filename,
+		 postscript_filename);
+  }
   (void) MagickMonitorFormatted(0,8,&image->exception,RenderPostscriptText,
-                                image->filename);
-  status=InvokePostscriptDelegate(image_info->verbose,command);
+                                image_info->filename);
+  status=InvokePostscriptDelegate(image_info->verbose,command,exception);
   if (!IsAccessibleAndNotEmpty(image_info->filename))
     {
       /*
-        Append a showpage operator if needed.
+        Ghostscript requires a showpage operator.
       */
       file=fopen(postscript_filename,"ab");
       if (file == (FILE *) NULL)
@@ -342,42 +352,55 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
       (void) fputs("showpage\n",file);
       (void) fclose(file);
-      status=InvokePostscriptDelegate(image_info->verbose,command);
+      status=InvokePostscriptDelegate(image_info->verbose,command,exception);
     }
   (void) LiberateTemporaryFile(postscript_filename);
   (void) MagickMonitorFormatted(7,8,&image->exception,RenderPostscriptText,
-                                image->filename);
-  if (!IsAccessibleAndNotEmpty(image_info->filename))
+                                image_info->filename);
+  if (IsAccessibleAndNotEmpty(image_info->filename))
+    {
+      /*
+	Read Ghostscript output.
+      */
+      ImageInfo
+	*clone_info;
+
+      clone_info=CloneImageInfo(image_info);
+      clone_info->blob=(void *) NULL;
+      clone_info->length=0;
+      clone_info->magick[0]='\0';
+      image=ReadImage(clone_info,exception);
+      DestroyImageInfo(clone_info);
+    }
+  (void) LiberateTemporaryFile((char *) image_info->filename);
+#if defined(HasDPS)
+  if (image == (Image *) NULL)
     {
       /*
         Ghostscript has failed-- try the Display Postscript Extension.
       */
-      (void) LiberateTemporaryFile((char *) image_info->filename);
       (void) FormatString((char *) image_info->filename,"dps:%.1024s",filename);
       image=ReadImage(image_info,exception);
-      if (image != (Image *) NULL)
-        return(image);
-      ThrowReaderException(DelegateError,PostscriptDelegateFailed,image)
     }
-  clone_info=CloneImageInfo(image_info);
-  clone_info->blob=(void *) NULL;
-  clone_info->length=0;
-  clone_info->magick[0]='\0';
-  image=ReadImage(clone_info,exception);
-  DestroyImageInfo(clone_info);
-  (void) LiberateTemporaryFile((char *) image_info->filename);
+#endif /* defined(HasDPS) */
   if (image == (Image *) NULL)
-    ThrowReaderException(DelegateError,PostscriptDelegateFailed,image);
-  do
-  {
-    (void) strcpy(image->magick,"PS");
-    (void) strlcpy(image->filename,filename,MaxTextExtent);
-    next_image=SyncNextImageInList(image);
-    if (next_image != (Image *) NULL)
-      image=next_image;
-  } while (next_image != (Image *) NULL);
-  while (image->previous != (Image *) NULL)
-    image=image->previous;
+    {
+      if (UndefinedException == exception->severity)
+	ThrowException(exception,DelegateError,PostscriptDelegateFailed,filename);
+    }
+  else
+    {
+      do
+	{
+	  (void) strcpy(image->magick,"PS");
+	  (void) strlcpy(image->filename,filename,MaxTextExtent);
+	  next_image=SyncNextImageInList(image);
+	  if (next_image != (Image *) NULL)
+	    image=next_image;
+	} while (next_image != (Image *) NULL);
+      while (image->previous != (Image *) NULL)
+	image=image->previous;
+    }
   return(image);
 }
 
@@ -1029,12 +1052,6 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
             Image
               *preview_image;
 
-            long
-              y;
-
-            register long
-              x;
-
             /*
               Create preview image.
             */
@@ -1237,7 +1254,8 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(y,image->rows))
                   if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                              SaveImageText,image->filename))
+                                              SaveImageText,image->filename,
+					      image->columns,image->rows))
                     break;
             } 
             if (bp != buffer)
@@ -1249,9 +1267,6 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
           }
         else
           {
-            int
-              y;
-
             /*
               Dump image as bitmap.
             */
@@ -1309,7 +1324,8 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(y,image->rows))
                   if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                              SaveImageText,image->filename))
+                                              SaveImageText,image->filename,
+					      image->columns,image->rows))
                     break;
             }
             if (bp != buffer)
@@ -1375,7 +1391,8 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                 if (image->previous == (Image *) NULL)
                   if (QuantumTick(y,image->rows))
                     if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                                SaveImageText,image->filename))
+                                                SaveImageText,image->filename,
+						image->columns,image->rows))
                       break;
               }
               if (bp != buffer)
@@ -1421,7 +1438,8 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                 if (image->previous == (Image *) NULL)
                   if (QuantumTick(y,image->rows))
                     if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                                SaveImageText,image->filename))
+                                                SaveImageText,image->filename,
+						image->columns,image->rows))
                       break;
               }
               if (bp != buffer)
@@ -1506,7 +1524,8 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                 if (image->previous == (Image *) NULL)
                   if (QuantumTick(y,image->rows))
                     if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                                SaveImageText,image->filename))
+                                                SaveImageText,image->filename,
+						image->columns,image->rows))
                       break;
               }
               if (bp != buffer)
@@ -1547,7 +1566,8 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                 if (image->previous == (Image *) NULL)
                   if (QuantumTick(y,image->rows))
                     if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                                SaveImageText,image->filename))
+                                                SaveImageText,image->filename,
+						image->columns,image->rows))
                       break;
               }
               if (bp != buffer)
