@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 GraphicsMagick Group
+% Copyright (C) 2003 - 2010 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -112,7 +112,6 @@ MagickExport void DestroyDelegateInfo(void)
   register DelegateInfo
     *p;
 
-  AcquireSemaphoreInfo(&delegate_semaphore);
   for (p=delegate_list; p != (DelegateInfo *) NULL; )
   {
     delegate_info=p;
@@ -128,7 +127,6 @@ MagickExport void DestroyDelegateInfo(void)
     MagickFreeMemory(delegate_info);
   }
   delegate_list=(DelegateInfo *) NULL;
-  LiberateSemaphoreInfo(&delegate_semaphore);
   DestroySemaphoreInfo(&delegate_semaphore);
 }
 
@@ -256,17 +254,17 @@ MagickExport const DelegateInfo *GetDelegateInfo(const char *decode,
 
   if (delegate_list == (DelegateInfo *) NULL)
     {
-      AcquireSemaphoreInfo(&delegate_semaphore);
+      LockSemaphoreInfo(delegate_semaphore);
       if (delegate_list == (DelegateInfo *) NULL)
         (void) ReadConfigureFile(DelegateFilename,0,exception);
-      LiberateSemaphoreInfo(&delegate_semaphore);
+      UnlockSemaphoreInfo(delegate_semaphore);
     }
   if ((LocaleCompare(decode,"*") == 0) && (LocaleCompare(encode,"*") == 0))
     return((const DelegateInfo *) delegate_list);
   /*
     Search for requested delegate.
   */
-  AcquireSemaphoreInfo(&delegate_semaphore);
+  LockSemaphoreInfo(delegate_semaphore);
   for (p=delegate_list; p != (const DelegateInfo *) NULL; p=p->next)
   {
     if (p->mode > 0)
@@ -306,7 +304,7 @@ MagickExport const DelegateInfo *GetDelegateInfo(const char *decode,
         delegate_list->previous=p;
         delegate_list=p;
       }
-  LiberateSemaphoreInfo(&delegate_semaphore);
+  UnlockSemaphoreInfo(delegate_semaphore);
   return((const DelegateInfo *) p);
 }
 
@@ -359,6 +357,10 @@ MagickExport const DelegateInfo *GetPostscriptDelegateInfo(const ImageInfo *imag
     {
       (void) strlcpy(delegate,"gs-gray",sizeof(delegate));
     }
+  else if (PaletteType == image_info->type)
+    {
+      (void) strlcpy(delegate,"gs-palette",sizeof(delegate));
+    }
   else if ((GrayscaleMatteType == image_info->type) ||
            (PaletteMatteType == image_info->type) ||
            (TrueColorMatteType == image_info->type))
@@ -373,6 +375,33 @@ MagickExport const DelegateInfo *GetPostscriptDelegateInfo(const ImageInfo *imag
 %                                                                             %
 %                                                                             %
 %                                                                             %
++   I n i t i a l i z e D e l e g a t e I n f o                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  Method InitializeDelegateInfo initializes the delegate facility
+%
+%  The format of the InitializeDelegateInfo method is:
+%
+%      MagickPassFail InitializeDelegateInfo(void)
+%
+%
+*/
+MagickPassFail
+InitializeDelegateInfo(void)
+{
+  assert(delegate_semaphore == (SemaphoreInfo *) NULL);
+  delegate_semaphore=AllocateSemaphoreInfo();
+  return MagickPass;
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   I n v o k e D e l e g a t e                                               %
 %                                                                             %
 %                                                                             %
@@ -444,6 +473,58 @@ UnixShellTextEscape(char *dst, const char *src, const size_t size)
 }
 #endif /* POSIX */
 
+#if defined(MSWINDOWS)
+static size_t
+WindowsShellTextEscape(char *dst, const char *src, const size_t size)
+{
+  size_t
+    length=0;
+
+  char
+    *p;
+
+  const char
+    *q;
+
+  assert(dst != NULL);
+  assert(src != (const char *) NULL);
+  assert(size >= 1);
+
+
+  /*
+    Copy src to dst within bounds of size-1, while escaping special
+    characters.
+  */
+  for ( p=dst, q=src, length=0 ;
+        (*q != 0) && (length < size-1) ;
+        length++, p++, q++ )
+    {
+      register const char c = *q;
+#if 0
+      /*
+	FIXME: Currently the correct implementation is not known so we
+	don't alter arguments at the moment.
+      */
+      if ((c == '\\') ||
+          (c == '"') ||
+          (c == '%%'))
+        {
+          if (length+1 >= size-1)
+            break;
+          *p = '\\';
+          p++;
+          length++;
+        }
+#endif
+      *p = c;
+    }
+
+  dst[length]='\0';
+
+  return length;
+}
+#endif /* MSWINDOWS */
+
 MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
   const char *decode,const char *encode,ExceptionInfo *exception)
 {
@@ -511,7 +592,7 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
         (encode && (delegate_info->decode != (char *) NULL)))
       {
         char
-          filename[MaxTextExtent],
+          decode_filename[MaxTextExtent],
           *magick;
 
         ImageInfo
@@ -558,15 +639,15 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
         (void) strlcpy((char *) clone_info->magick,magick,MaxTextExtent);
         (void) strlcpy(image->magick,magick,MaxTextExtent);
         MagickFreeMemory(magick);
-        (void) strlcpy(filename,image->filename,MaxTextExtent);
+        (void) strlcpy(decode_filename,image->filename,MaxTextExtent);
         FormatString(clone_info->filename,"%.1024s:",delegate_info->decode);
-        (void) SetImageInfo(clone_info,True,exception);
+        (void) SetImageInfo(clone_info,SETMAGICK_WRITE,exception);
         (void) strlcpy(clone_info->filename,image_info->filename,
           MaxTextExtent);
         for (p=image; p != (Image *) NULL; p=p->next)
         {
           FormatString(p->filename,"%.1024s:%.1024s",delegate_info->decode,
-            filename);
+            decode_filename);
           status=WriteImage(clone_info,p);
           if (status == False)
             {
@@ -618,7 +699,6 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
       status=False;
       goto error_exit;
     }
-#if defined(POSIX)
     {
       MagickBool
         needs_shell;
@@ -688,7 +768,14 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
             Expand sprintf-style codes in delegate command to command
             string, escaping replacement text appropriately
           */
-          command=TranslateTextEx(image_info,image,commands[i],UnixShellTextEscape);
+          command=TranslateTextEx(image_info,image,commands[i],
+#if defined(POSIX)
+				  UnixShellTextEscape
+#endif /* POSIX */
+#if defined(MSWINDOWS)
+				  WindowsShellTextEscape
+#endif /* MSWINDOWS */
+				  );
           if (command == (char *) NULL)
             break;
           /*
@@ -697,20 +784,6 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
           status=SystemCommand(image_info->verbose,command);
         }
     }
-#else
-    {
-      /*
-        Expand sprintf-style codes in delegate command to command string
-      */
-      command=TranslateText(image_info,image,commands[i]);
-      if (command == (char *) NULL)
-        break;
-      /*
-        Execute delegate using command shell.
-      */
-      status=SystemCommand(image_info->verbose,command);
-    }
-#endif
     MagickFreeMemory(command);
     /* Liberate convenience temporary files */
     (void) LiberateTemporaryFile(image_info->unique);
@@ -740,7 +813,7 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   I n v o k e P o s t s r i p t D e l e g a t e                             %
++   I n v o k e P o s t s c r i p t D e l e g a t e                           %
 %                                                                             %
 %                                                                             %
 %                                                                             %
@@ -751,13 +824,13 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
 %
 %  The format of the InvokePostscriptDelegate method is:
 %
-%      unsigned int InvokePostscriptDelegate(const unsigned int verbose,
-%        const char *command)
+%      MagickPassFail InvokePostscriptDelegate(const unsigned int verbose,
+%        const char *command, ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method InvokePostscriptDelegate returns True is the command
-%      is successfully executed, otherwise False.
+%    o status:  Method InvokePostscriptDelegate returns MagickPass if the command
+%      is successfully executed, otherwise MagickFail.
 %
 %    o verbose: A value other than zero displays the command prior to
 %      executing it.
@@ -768,23 +841,29 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
 %
 %
 */
-MagickExport unsigned int InvokePostscriptDelegate(const unsigned int verbose,
-  const char *command)
+MagickExport MagickPassFail
+InvokePostscriptDelegate(const unsigned int verbose,
+			 const char *command,ExceptionInfo *exception)
 {
-#if defined(HasGS) || defined(MSWINDOWS)
+  register long
+    i;
+
   char
     **argv;
 
+  int
+    argc;
+
+  int
+    status;
+
+#if defined(HasGS) || defined(MSWINDOWS)
+
   gs_main_instance
     *interpreter;
 
   int
-    argc,
-    pexit_code,
-    status;
-
-  register long
-    i;
+    pexit_code;
 
 #if defined(MSWINDOWS)
   const GhostscriptVectors
@@ -805,72 +884,99 @@ MagickExport unsigned int InvokePostscriptDelegate(const unsigned int verbose,
   gs_func_struct.run_string=gsapi_run_string;
   gs_func_struct.delete_instance=gsapi_delete_instance;
 #endif
-  if (gs_func == (GhostscriptVectors *) NULL)
+  if (gs_func != (GhostscriptVectors *) NULL)
     {
-#if defined(POSIX)
-      {
-        int arg_count;
-        char **arg_array;
-        arg_array = StringToArgv(command,&arg_count);
-        return MagickSpawnVP(verbose,arg_array[1],arg_array+1);
-      }
+
+      /*
+	Allocate an interpreter.
+      */
+      interpreter = (gs_main_instance *) NULL;
+      status=(gs_func->new_instance)(&interpreter,(void *) NULL);
+      if (status < 0)
+	{
+	  ThrowException(exception,DelegateError,
+			 FailedToAllocateGhostscriptInterpreter,command);
+	  return(MagickFail);
+	}
+      /*
+	Initialize interpreter with argument list.
+      */
+      argv=StringToArgv(command,&argc);
+      if (argv == (char **) NULL)
+	{
+	  ThrowException(exception,DelegateError,FailedToAllocateArgumentList,
+			 command);
+	  return(MagickFail);
+	}
+
+      if (verbose)
+	{
+	  char
+	    buffer[MaxTextExtent];
+
+#if defined(MSWINDOWS)
+	  (void) NTGhostscriptDLL(buffer,sizeof(buffer));
 #else
-      return(SystemCommand(verbose,command));
+	  (void) strlcpy(buffer,"[ghostscript library]",sizeof(buffer));
 #endif
-    }
-  if (verbose)
-    {
-      (void) fputs("[ghostscript library]",stdout);
-      (void) fputs(strchr(command,' '),stdout);
-    }
-  /*
-    Allocate an interpreter.
-  */
-  interpreter = (gs_main_instance *) NULL;
-  status=(gs_func->new_instance)(&interpreter,(void *) NULL);
-  if (status < 0)
-    {
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                            "Failed to allocate Ghostscript interpreter!");
-      return(False);
-    }
-  /*
-    Initialize interpreter with argument list.
-  */
-  argv=StringToArgv(command,&argc);
-  if (argv == (char **) NULL)
-    {
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                            "Failed to allocate Ghostscript argument list!");
-      return(False);
-    }
-  status=(gs_func->init_with_args)(interpreter,argc-1,argv+1);
-  if (status == 0)
-    {
-      status=(gs_func->run_string)
-        (interpreter,"systemdict /start get exec\n",0,&pexit_code);
+	  (void) fputs(buffer,stderr);
+	  for (i=2 ; i < argc ; i++)
+	    (void) fprintf(stderr," \"%s\"",argv[i]);
+	  (void) fflush(stderr);
+	}
+      status=(gs_func->init_with_args)(interpreter,argc-1,argv+1);
+      if (status == 0)
+	{
+	  status=(gs_func->run_string)
+	    (interpreter,"systemdict /start get exec\n",0,&pexit_code);
+	  if ((status == 0) || (status <= -100))
+	    {
+	      char
+		reason[MaxTextExtent];
+
+	      FormatString(reason,"Ghostscript returns status %d, exit code %d",
+			   status,pexit_code);
+	      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"%s",reason);
+	      ThrowException(exception,DelegateError,PostscriptDelegateFailed,command);
+	    }
+	}
+      /*
+	Exit interpreter.
+      */
+      (gs_func->exit)(interpreter);
+      /*
+	Deallocate interpreter.
+      */
+      (gs_func->delete_instance)(interpreter);
+      for (i=0; i < argc; i++)
+	MagickFreeMemory(argv[i]);
+      MagickFreeMemory(argv);
       if ((status == 0) || (status <= -100))
-        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                              "Ghostscript returns status %d, exit code %d",
-                              status,pexit_code);
+	return(MagickFail);
+      return(MagickPass);
     }
-  /*
-    Exit interpreter.
-  */
-  (gs_func->exit)(interpreter);
-  /*
-    Deallocate interpreter.
-  */
-  (gs_func->delete_instance)(interpreter);
-  for (i=0; i < argc; i++)
-    MagickFreeMemory(argv[i]);
-  MagickFreeMemory(argv);
-  if ((status == 0) || (status <= -100))
-    return(False);
-  return(True);
-#else
-  return(SystemCommand(verbose,command));
-#endif
+#endif /* defined(HasGS) || defined(MSWINDOWS) */
+
+  status=MagickFail;
+  {
+    argv = StringToArgv(command,&argc);
+    if (argv == (char **) NULL)
+      {
+	ThrowException(exception,DelegateError,
+		       FailedToAllocateArgumentList,
+		       command);
+      }
+    else
+      {
+	if (MagickSpawnVP(verbose,argv[1],argv+1) == 0)
+	  status=MagickPass;
+	for (i=0; i < argc; i++)
+	  MagickFreeMemory(argv[i]);
+	MagickFreeMemory(argv);
+      }
+  }
+
+  return status;
 }
 
 /*
@@ -913,7 +1019,7 @@ MagickExport unsigned int ListDelegateInfo(FILE *file,ExceptionInfo *exception)
   if (file == (const FILE *) NULL)
     file=stdout;
   (void) GetDelegateInfo("*","*",exception);
-  AcquireSemaphoreInfo(&delegate_semaphore);
+  LockSemaphoreInfo(delegate_semaphore);
   for (p=delegate_list; p != (const DelegateInfo *) NULL; p=p->next)
   {
     if ((p->previous == (DelegateInfo *) NULL) ||
@@ -952,7 +1058,7 @@ MagickExport unsigned int ListDelegateInfo(FILE *file,ExceptionInfo *exception)
       /* Format output so that command spans multiple lines if
          necessary */
       if (getenv("COLUMNS"))
-        screen_width=atoi(getenv("COLUMNS"))-1;
+        screen_width=MagickAtoI(getenv("COLUMNS"))-1;
       command_length=strlen(commands[0]);
       command_start_column=fprintf(file,"%8s%c=%c%s  ",p->decode ? p->decode : "",
         p->mode <= 0 ? '<' : ' ',p->mode >= 0 ? '>' : ' ',delegate);
@@ -978,9 +1084,10 @@ MagickExport unsigned int ListDelegateInfo(FILE *file,ExceptionInfo *exception)
     }
     for (i=0; commands[i] != (char *) NULL; i++)
       MagickFreeMemory(commands[i]);
+    MagickFreeMemory(commands);
   }
   (void) fflush(file);
-  LiberateSemaphoreInfo(&delegate_semaphore);
+  UnlockSemaphoreInfo(delegate_semaphore);
   return(True);
 }
 
@@ -1018,8 +1125,8 @@ MagickExport unsigned int ListDelegateInfo(FILE *file,ExceptionInfo *exception)
 */
 #if defined(MSWINDOWS)
 static void CatDelegatePath(char *path,
-                               const char *binpath,
-                               const char *command)
+			    const char *binpath,
+			    const char *command)
 {
   strcpy(path,binpath);
   strcat(path,command);
