@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2008 GraphicsMagick Group
+% Copyright (C) 2008 - 2009 GraphicsMagick Group
 %
 % This program is covered by multiple licenses, which are described in
 % Copyright.txt. You should have received a copy of Copyright.txt with this
@@ -21,7 +21,7 @@
 MagickExport const char *BlobModeToString(BlobMode blob_mode)
 {
   const char
-    *mode_string="Undefined";
+    *mode_string="?";
 
   switch (blob_mode)
   {
@@ -50,11 +50,12 @@ MagickExport const char *BlobModeToString(BlobMode blob_mode)
 MagickExport const char *ChannelTypeToString(const ChannelType channel)
 {
   const char
-    *channel_type="undefined";
+    *channel_type="?";
 
   switch (channel)
     {
     case UndefinedChannel:
+      channel_type="undefined";
       break;
     case RedChannel:
       channel_type="red";
@@ -132,7 +133,7 @@ MagickExport ChannelType StringToChannelType(const char *option)
 MagickExport const char* ClassTypeToString(const ClassType class_type)
 {
   const char
-    *log_class_type="Unknown";
+    *log_class_type="?";
 
   switch (class_type)
     {
@@ -155,7 +156,7 @@ MagickExport const char* ClassTypeToString(const ClassType class_type)
 MagickExport const char *ColorspaceTypeToString(const ColorspaceType colorspace)
 {
   const char
-    *colorspace_type = "Unknown";
+    *colorspace_type = "?";
   
   switch (colorspace)
     {
@@ -277,7 +278,7 @@ MagickExport ColorspaceType StringToColorspaceType(const char *option)
 MagickExport const char *CompositeOperatorToString(const CompositeOperator composite_op)
 {
   const char
-    *composite_op_text = "Undefined";
+    *composite_op_text = "?";
 
   switch (composite_op)
     {
@@ -486,7 +487,7 @@ MagickExport CompositeOperator StringToCompositeOperator(const char *option)
 MagickExport const char* CompressionTypeToString(const CompressionType compression_type)
 {
   const char
-    *compression_string="Unknown";
+    *compression_string="?";
 
   switch (compression_type)
     {
@@ -532,6 +533,10 @@ MagickExport CompressionType StringToCompressionType(const char *option)
     compression_type=NoCompression;
   else if (LocaleCompare("BZip",option) == 0)
     compression_type=BZipCompression;
+  else if (LocaleCompare("BZip2",option) == 0)
+    compression_type=BZipCompression;
+  else if (LocaleCompare("BZ2",option) == 0)
+    compression_type=BZipCompression;
   else if (LocaleCompare("Fax",option) == 0)
     compression_type=FaxCompression;
   else if (LocaleCompare("Group4",option) == 0)
@@ -546,13 +551,74 @@ MagickExport CompressionType StringToCompressionType(const char *option)
     compression_type=RLECompression;
   else if (LocaleCompare("Zip",option) == 0)
     compression_type=ZipCompression;
+  else if (LocaleCompare("GZip",option) == 0)
+    compression_type=ZipCompression;
 
   return compression_type;
 }
 
+/*
+  ConfirmAccessMode
+*/
+MagickExport const char *ConfirmAccessModeToString(const ConfirmAccessMode access_mode)
+{
+  const char
+    *mode_string="?";
+
+  switch(access_mode)
+    {
+    case UndefinedConfirmAccessMode:
+      mode_string="Undefined";
+      break;
+    case FileExecuteConfirmAccessMode:
+      mode_string="Execute";
+      break;
+    case FileReadConfirmAccessMode:
+      mode_string="Read";
+      break;
+    case FileWriteConfirmAccessMode:
+      mode_string="Write";
+      break;
+    case URLGetFTPConfirmAccessMode:
+      mode_string="FTP Get";
+      break;
+    case URLGetFileConfirmAccessMode:
+      mode_string="FILE Get";
+      break;
+    case URLGetHTTPConfirmAccessMode:
+      mode_string="HTTP Get";
+      break;
+    }
+
+  return mode_string;
+}
+
 /*
   EndianType
 */
+MagickExport const char *EndianTypeToString(const EndianType endian_type)
+{
+  const char
+    *endian_string="?";
+
+  switch (endian_type)
+    {
+    case UndefinedEndian:
+      endian_string="Undefined";
+      break;
+    case LSBEndian:
+      endian_string="LSB";
+      break;
+    case MSBEndian:
+      endian_string="MSB";
+      break;
+    case NativeEndian:
+      endian_string="Native";
+      break;
+    }
+
+  return endian_string;
+}
 MagickExport EndianType StringToEndianType(const char *option)
 {
   EndianType
@@ -657,11 +723,12 @@ MagickExport const char *
 HighlightStyleToString(const HighlightStyle difference_style)
 {
   const char
-    *style="Undefined";;
+    *style="?";
 
   switch(difference_style)
     {
     case UndefinedHighlightStyle:
+      style="Undefined";
       break;
     case AssignHighlightStyle:
       style="Assign";
@@ -702,7 +769,7 @@ StringToHighlightStyle(const char *option)
 MagickExport const char *ImageTypeToString(const ImageType image_type)
 {
   const char
-    *p = "Unknown";
+    *p = "?";
 
   switch (image_type)
     {
@@ -775,6 +842,32 @@ MagickExport ImageType StringToImageType(const char *option)
 /*
   InterlaceType
 */
+MagickExport const char *InterlaceTypeToString(const InterlaceType interlace_type)
+{
+  const char
+    *interlace_string = "?";
+
+  switch (interlace_type)
+    {
+    case UndefinedInterlace:
+      interlace_string = "Undefined";
+      break;
+    case NoInterlace:
+      interlace_string = "No";
+      break;
+    case LineInterlace:
+      interlace_string = "Line";
+      break;
+    case PlaneInterlace:
+      interlace_string = "Plane";
+      break;
+    case PartitionInterlace:
+      interlace_string = "Partition";
+      break;
+    }
+
+  return interlace_string;
+}
 MagickExport InterlaceType StringToInterlaceType(const char *option)
 {
   InterlaceType
@@ -798,7 +891,7 @@ MagickExport InterlaceType StringToInterlaceType(const char *option)
 MagickExport const char *MapModeToString(MapMode map_mode)
 {
   char
-    *mode_string="Undefined";
+    *mode_string="?";
 
   switch (map_mode)
   {
@@ -877,7 +970,7 @@ MagickExport MetricType StringToMetricType(const char *option)
 MagickExport const char *OrientationTypeToString(const OrientationType orientation_type)
 {
   const char *
-    orientation = "Unknown";
+    orientation = "?";
 
   switch (orientation_type)
     {
@@ -1062,11 +1155,12 @@ MagickExport QuantumOperator StringToQuantumOperator(const char *option)
 MagickExport const char *QuantumOperatorToString(const QuantumOperator quantum_operator)
 {
   const char
-    *operator_text = "undefined";
+    *operator_text = "?";
 
   switch (quantum_operator)
     {
     case UndefinedQuantumOp:
+      operator_text="undefined";
       break;
     case AddQuantumOp:
       operator_text="add";
@@ -1157,7 +1251,7 @@ MagickExport const char *QuantumOperatorToString(const QuantumOperator quantum_o
 MagickExport const char *ResizeFilterToString(const FilterTypes filter)
 {
   const char *
-    filter_string = "Unknown";
+    filter_string = "?";
 
   switch (filter)
     {
@@ -1226,12 +1320,16 @@ MagickExport ResourceType StringToResourceType(const char *option)
     resource_type=DiskResource;
   else if (LocaleCompare("File",option) == 0)
     resource_type=FileResource;
+  else if (LocaleCompare("Files",option) == 0)
+    resource_type=FileResource;
   else if (LocaleCompare("Map",option) == 0)
     resource_type=MapResource;
   else if (LocaleCompare("Memory",option) == 0)
     resource_type=MemoryResource;
   else if (LocaleCompare("Pixels",option) == 0)
     resource_type=PixelsResource;
+  else if (LocaleCompare("Threads",option) == 0)
+    resource_type=ThreadsResource;
   return resource_type;
 }
 
@@ -1241,7 +1339,7 @@ MagickExport ResourceType StringToResourceType(const char *option)
 MagickExport const char *StorageTypeToString(const StorageType storage_type)
 {
     const char
-    *p = "Unknown";
+    *p = "?";
 
     switch (storage_type)
       {
@@ -1274,7 +1372,7 @@ MagickExport const char *StorageTypeToString(const StorageType storage_type)
 MagickExport const char *QuantumSampleTypeToString(const QuantumSampleType sample_type)
 {
     const char
-    *p = "Unknown";
+    *p = "?";
 
     switch (sample_type)
       {
@@ -1336,7 +1434,7 @@ MagickExport const char *StyleTypeToString(StyleType style)
 MagickExport const char *QuantumTypeToString(const QuantumType quantum_type)
 {
   const char
-    *p = "Unknown";
+    *p = "?";
 
   switch (quantum_type)
     {
