@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2003 GraphicsMagick Group
+  Copyright (C) 2003-2010 GraphicsMagick Group
   Copyright (C) 2002 ImageMagick Studio
  
   This program is covered by multiple licenses, which are described in
@@ -20,19 +20,36 @@ typedef struct _SemaphoreInfo SemaphoreInfo;
 extern MagickExport SemaphoreInfo
   *AllocateSemaphoreInfo(void);
 
-extern MagickExport unsigned int
+extern MagickExport void
+  DestroySemaphoreInfo(SemaphoreInfo **),
   LockSemaphoreInfo(SemaphoreInfo *),
   UnlockSemaphoreInfo(SemaphoreInfo *);
 
+
+/*
+  These are deprecated.
+*/
+extern MagickExport void
+  AcquireSemaphoreInfo(SemaphoreInfo **) MAGICK_FUNC_DEPRECATED,
+  LiberateSemaphoreInfo(SemaphoreInfo **) MAGICK_FUNC_DEPRECATED;
+
+/*
+  These should not be MagickExport.
+*/
 extern MagickExport void
-  AcquireSemaphoreInfo(SemaphoreInfo **),
   DestroySemaphore(void),
-  DestroySemaphoreInfo(SemaphoreInfo **),
-  InitializeSemaphore(void),
-  LiberateSemaphoreInfo(SemaphoreInfo **);
+  InitializeSemaphore(void);
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
 #endif
 
 #endif
+
+/*
+ * Local Variables:
+ * mode: c
+ * c-basic-offset: 2
+ * fill-column: 78
+ * End:
+ */
