@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003, 2007 GraphicsMagick Group
+% Copyright (C) 2003 - 2010 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -81,8 +81,9 @@
 %
 %
 */
-MagickExport Image *AffineTransformImage(const Image *image,
-  const AffineMatrix *affine,ExceptionInfo *exception)
+MagickExport Image *
+AffineTransformImage(const Image *image,const AffineMatrix *affine,
+		     ExceptionInfo *exception)
 {
   AffineMatrix
     transform;
@@ -172,7 +173,7 @@ MagickExport Image *AffineTransformImage(const Image *image,
 %
 %  The format of the CropToFitImage method is:
 %
-%      Image *CropToFitImage(Image **image,const double x_shear,
+%      MagickPassFail CropToFitImage(Image **image,const double x_shear,
 %        const double x_shear,const double width,const double height,
 %        const unsigne int rotate,ExceptionInfo *exception)
 %
@@ -186,9 +187,11 @@ MagickExport Image *AffineTransformImage(const Image *image,
 %
 %
 */
-static void CropToFitImage(Image **image,const double x_shear,
-  const double y_shear,const double width,const double height,
-  const unsigned int rotate,ExceptionInfo *exception)
+static MagickPassFail
+CropToFitImage(Image **image,
+	       const double x_shear,const double y_shear,
+	       const double width,const double height,
+	       const unsigned int rotate,ExceptionInfo *exception)
 {
   Image
     *crop_image;
@@ -243,11 +246,11 @@ static void CropToFitImage(Image **image,const double x_shear,
   geometry.y=(long) ceil(min.y-0.5);
   crop_image=CropImage(*image,&geometry,exception);
   if (crop_image != (Image *) NULL)
-    {
-      crop_image->page=(*image)->page;
-      DestroyImage(*image);
-      *image=crop_image;
-    }
+    crop_image->page=(*image)->page;
+  DestroyImage(*image);
+  *image=crop_image;
+
+  return (*image != (Image *) NULL ? MagickPass : MagickFail);
 }
 
 /*
@@ -279,10 +282,18 @@ static void CropToFitImage(Image **image,const double x_shear,
 %
 %    o rotations: Specifies the number of 90 degree rotations.
 %
+%    o exception: Return any errors or warnings in this structure.
+%
 %
 */
-static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
-                                  ExceptionInfo *exception)
+#if 1
+#if !defined(DisableSlowOpenMP)
+#  define IntegralRotateImageUseOpenMP
+#endif
+#endif
+static Image *
+IntegralRotateImage(const Image *image,unsigned int rotations,
+		    ExceptionInfo *exception)
 {
   char
     message[MaxTextExtent];
@@ -294,8 +305,8 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
     page;
 
   long
-    tile_width_max=128,
-    tile_height_max=128;
+    tile_width_max,
+    tile_height_max;
 
   MagickPassFail
     status=MagickPass;
@@ -306,37 +317,68 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
   assert(image != (Image *) NULL);
   page=image->page;
   rotations%=4;
+
+  {
+    /*
+      Clone appropriately to create rotate image.
+    */
+    unsigned long
+      clone_columns=0,
+      clone_rows=0;
+    
+    switch (rotations)
+      {
+      case 0:
+	clone_columns=0;
+	clone_rows=0;
+	break;
+      case 2:
+	clone_columns=image->columns;
+	clone_rows=image->rows;
+	break;
+      case 1:
+      case 3:
+	clone_columns=image->rows;
+	clone_rows=image->columns;
+	break;
+      }
+    rotate_image=CloneImage(image,clone_columns,clone_rows,True,exception);
+    if (rotate_image == (Image *) NULL)
+      return((Image *) NULL);
+    if (rotations != 0)
+      if (ModifyCache(rotate_image,exception) != MagickPass)
+	{
+	  DestroyImage(rotate_image);
+	  return (Image *) NULL;
+	}
+  }
+
+  tile_height_max=tile_width_max=2048/sizeof(PixelPacket); /* 2k x 2k = 4MB */
   if ((rotations == 1) || (rotations == 3))
     {
-      rotate_image=CloneImage(image,image->rows,image->columns,True,exception);
+      /*
+	Allow override of tile geometry for testing.
+      */
+      const char *
+	value;
 
-      {
-        const char *
-          value;
+      if (!GetPixelCacheInCore(image) || !GetPixelCacheInCore(rotate_image))
+	tile_height_max=tile_width_max=8192/sizeof(PixelPacket); /* 8k x 8k = 64MB */
 
-        /*
-          Allow override of tile geometry for testing.
-        */
-        if ((value=getenv("MAGICK_ROTATE_TILE_GEOMETRY")))
-          {
-            double
-              width,
-              height;
-            
-            if (GetMagickDimension(value,&width,&height,NULL,NULL) == 2)
-              {
-                tile_height_max=(unsigned long) height;
-                tile_width_max=(unsigned long) width;
-              }
-          }
-      }
+      if ((value=getenv("MAGICK_ROTATE_TILE_GEOMETRY")))
+	{
+	  double
+	    width,
+	    height;
+	  
+	  if (GetMagickDimension(value,&width,&height,NULL,NULL) == 2)
+	    {
+	      tile_height_max=(unsigned long) height;
+	      tile_width_max=(unsigned long) width;
+	    }
+	}
     }
-  else
-    {
-      rotate_image=CloneImage(image,image->columns,image->rows,True,exception);
-    }
-  if (rotate_image == (Image *) NULL)
-    return((Image *) NULL);
+
   /*
     Integral rotate the image.
   */
@@ -345,70 +387,12 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
     case 0:
       {
         /*
-          Rotate 0 degrees.
+          Rotate 0 degrees (nothing more to do).
         */
-        long
-          y;
-
-        unsigned long
-          row_count=0;
-
-        (void) strlcpy(message,"[%s] Rotate image 0 degrees...",sizeof(message));
-#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
-#  pragma omp parallel for schedule(static,8) shared(row_count, status)
-#endif
-        for (y=0; y < (long) image->rows; y++)
-          {
-            register const PixelPacket
-              *p;
-            
-            register PixelPacket
-              *q;
-            
-            register const IndexPacket
-              *indexes;
-            
-            IndexPacket
-              *rotate_indexes;
-
-            MagickPassFail
-              thread_status;
-
-            thread_status=status;
-            if (thread_status == MagickFail)
-              continue;
-
-            p=AcquireImagePixels(image,0,y,image->columns,1,exception);
-            q=SetImagePixelsEx(rotate_image,0,y,rotate_image->columns,1,exception);
-            if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
-              thread_status=MagickFail;
-            if (thread_status != MagickFail)
-              {
-                (void) memcpy(q,p,image->columns*sizeof(PixelPacket));
-                indexes=AccessImmutableIndexes(image);
-                rotate_indexes=AccessMutableIndexes(rotate_image);
-                if ((indexes != (IndexPacket *) NULL) &&
-                    (rotate_indexes != (IndexPacket *) NULL))
-                  (void) memcpy(rotate_indexes,indexes,image->columns*
-                                sizeof(IndexPacket));
-
-                if (!SyncImagePixelsEx(rotate_image,exception))
-                  thread_status=MagickFail;
-              }
-#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
-#  pragma omp critical
-#endif
-            {
-              row_count++;
-              if (QuantumTick(row_count,image->rows))
-                if (!MagickMonitorFormatted(row_count,image->rows,exception,
-                                            message,image->filename))
-                  thread_status=MagickFail;
-                  
-              if (thread_status == MagickFail)
-                status=MagickFail;
-            }
-          }
+	(void) strlcpy(message,"[%s] Rotate: 0 degrees...",sizeof(message));
+	if (!MagickMonitorFormatted(image->rows-1,image->rows,exception,
+				    message,image->filename))
+	  status=MagickFail;
         break;
       }
     case 1:
@@ -425,12 +409,15 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
         long
           tile_y;
 
-        (void) strlcpy(message,"[%s] Rotate image 90 degrees...",sizeof(message));
+        (void) strlcpy(message,"[%s] Rotate: 90 degrees...",sizeof(message));
         total_tiles=(((image->rows/tile_height_max)+1)*
                      ((image->columns/tile_width_max)+1));        
         tile=0;
-#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
-#  pragma omp parallel for schedule(static,1) shared(status, tile)
+
+#if defined(IntegralRotateImageUseOpenMP)
+#  if defined(HAVE_OPENMP)
+#    pragma omp parallel for schedule(static,1) shared(status, tile)
+#  endif
 #endif
         for (tile_y=0; tile_y < (long) image->rows; tile_y+=tile_height_max)
           {
@@ -555,8 +542,10 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
                       }
                   }
 
-#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
-#  pragma omp critical
+#if defined(IntegralRotateImageUseOpenMP)
+#  if defined(HAVE_OPENMP)
+#    pragma omp critical (GM_IntegralRotateImage)
+#  endif
 #endif
                 {
                   tile++;
@@ -586,9 +575,11 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
         unsigned long
           row_count=0;
 
-        (void) strlcpy(message,"[%s] Rotate image 180 degrees...",sizeof(message));
-#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
-#  pragma omp parallel for schedule(static,8) shared(row_count, status)
+        (void) strlcpy(message,"[%s] Rotate: 180 degrees...",sizeof(message));
+#if defined(IntegralRotateImageUseOpenMP)
+#  if defined(HAVE_OPENMP)
+#    pragma omp parallel for schedule(static,8) shared(row_count, status)
+#  endif
 #endif
         for (y=0; y < (long) image->rows; y++)
           {
@@ -633,8 +624,10 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
                 if (!SyncImagePixelsEx(rotate_image,exception))
                   thread_status=MagickFail;
               }
-#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
-#  pragma omp critical
+#if defined(IntegralRotateImageUseOpenMP)
+#  if defined(HAVE_OPENMP)
+#    pragma omp critical (GM_IntegralRotateImage)
+#  endif
 #endif
             {
               row_count++;
@@ -666,12 +659,14 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
         long
           tile_y;
 
-        (void) strlcpy(message,"[%s] Rotate image 270 degrees...",sizeof(message));
+        (void) strlcpy(message,"[%s] Rotate: 270 degrees...",sizeof(message));
         total_tiles=(((image->rows/tile_height_max)+1)*
                      ((image->columns/tile_width_max)+1));
         tile=0;
-#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
-#  pragma omp parallel for schedule(static,1) shared(status, tile)
+#if defined(IntegralRotateImageUseOpenMP)
+#  if defined(HAVE_OPENMP)
+#    pragma omp parallel for schedule(static,1) shared(status, tile)
+#  endif
 #endif
         for (tile_y=0; tile_y < (long) image->rows; tile_y+=tile_height_max)
           {
@@ -796,8 +791,10 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
                       }
                   }
 
-#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
-#  pragma omp critical
+#if defined(IntegralRotateImageUseOpenMP)
+#  if defined(HAVE_OPENMP)
+#    pragma omp critical (GM_IntegralRotateImage)
+#  endif
 #endif
                 {
                   tile++;
@@ -846,9 +843,9 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
 %
 %  The format of the XShearImage method is:
 %
-%      void XShearImage(Image *image,const double degrees,
+%      MagickPassFail XShearImage(Image *image,const double degrees,
 %        const unsigned long width,const unsigned long height,
-%        const long x_offset,long y_offset)
+%        const long x_offset,long y_offset,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
@@ -859,13 +856,16 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
 %    o width, height, x_offset, y_offset: Defines a region of the image
 %      to shear.
 %
+%    o exception: Return any errors or warnings in this structure.
+%
 */
 
-static void XShearImage(Image *image,const double degrees,
-                        const unsigned long width,const unsigned long height,
-                        const long x_offset,long y_offset)
+static MagickPassFail
+XShearImage(Image *image,const double degrees,
+	    const unsigned long width,const unsigned long height,
+	    const long x_offset,long y_offset,ExceptionInfo *exception)
 {
-#define XShearImageText  "[%s] X Shear image...  "
+#define XShearImageText  "[%s] X Shear: %+g degrees, region %lux%lu%+ld%+ld...  "
 
   long
     y;
@@ -928,7 +928,7 @@ static void XShearImage(Image *image,const double degrees,
           direction=LEFT;
         }
       step=(long) floor(displacement);
-      alpha=(double) MaxRGB*(displacement-step);
+      alpha=MaxRGBDouble*(displacement-step);
       if (alpha == 0.0)
         {
           /*
@@ -943,7 +943,7 @@ static void XShearImage(Image *image,const double degrees,
                 */
                 if (step > x_offset)
                   break;
-                p=GetImagePixelsEx(image,0,y+y_offset,image->columns,1,&image->exception);
+                p=GetImagePixelsEx(image,0,y+y_offset,image->columns,1,exception);
                 if (p == (PixelPacket *) NULL)
                   {
                     thread_status=MagickFail;
@@ -962,7 +962,7 @@ static void XShearImage(Image *image,const double degrees,
                 /*
                   Transfer pixels right-to-left.
                 */
-                p=GetImagePixelsEx(image,0,y+y_offset,image->columns,1,&image->exception);
+                p=GetImagePixelsEx(image,0,y+y_offset,image->columns,1,exception);
                 if (p == (PixelPacket *) NULL)
                   {
                     thread_status=MagickFail;
@@ -977,17 +977,19 @@ static void XShearImage(Image *image,const double degrees,
                 break;
               }
             }
-          if (!SyncImagePixelsEx(image,&image->exception))
+          if (!SyncImagePixelsEx(image,exception))
             thread_status=MagickFail;
 
 #if defined(HAVE_OPENMP)
-#  pragma omp critical
+#  pragma omp critical (GM_XShearImage)
 #endif
           {
             row_count++;
             if (QuantumTick(row_count,height))
-              if (!MagickMonitorFormatted(row_count,height,&image->exception,
-                                          XShearImageText,image->filename))
+              if (!MagickMonitorFormatted(row_count,height,exception,
+                                          XShearImageText,image->filename,
+					  degrees,width,height,
+					  x_offset,y_offset))
                 thread_status=MagickFail;
             
             if (thread_status == MagickFail)
@@ -1010,7 +1012,7 @@ static void XShearImage(Image *image,const double degrees,
             */
             if (step > x_offset)
               break;
-            p=GetImagePixelsEx(image,0,y+y_offset,image->columns,1,&image->exception);
+            p=GetImagePixelsEx(image,0,y+y_offset,image->columns,1,exception);
             if (p == (PixelPacket *) NULL)
               {
                 thread_status=MagickFail;
@@ -1041,7 +1043,7 @@ static void XShearImage(Image *image,const double degrees,
             /*
               Transfer pixels right-to-left.
             */
-            p=GetImagePixelsEx(image,0,y+y_offset,image->columns,1,&image->exception);
+            p=GetImagePixelsEx(image,0,y+y_offset,image->columns,1,exception);
             if (p == (PixelPacket *) NULL)
               {
                 thread_status=MagickFail;
@@ -1065,16 +1067,18 @@ static void XShearImage(Image *image,const double degrees,
             break;
           }
         }
-      if (!SyncImagePixelsEx(image,&image->exception))
+      if (!SyncImagePixelsEx(image,exception))
         thread_status=MagickFail;
 #if defined(HAVE_OPENMP)
-#  pragma omp critical
+#  pragma omp critical (GM_XShearImage)
 #endif
       {
         row_count++;
         if (QuantumTick(row_count,height))
-          if (!MagickMonitorFormatted(row_count,height,&image->exception,
-                                      XShearImageText,image->filename))
+          if (!MagickMonitorFormatted(row_count,height,exception,
+                                          XShearImageText,image->filename,
+					  degrees,width,height,
+					  x_offset,y_offset))
             thread_status=MagickFail;
         
         if (thread_status == MagickFail)
@@ -1083,6 +1087,8 @@ static void XShearImage(Image *image,const double degrees,
     }
   if (is_grayscale && IsGray(image->background_color))
     image->is_grayscale=True;
+
+  return status;
 }
 
 /*
@@ -1104,9 +1110,9 @@ static void XShearImage(Image *image,const double degrees,
 %
 %  The format of the YShearImage method is:
 %
-%      void YShearImage(Image *image,const double degrees,
+%      MagickPassFail YShearImage(Image *image,const double degrees,
 %        const unsigned long width,const unsigned long height,long x_offset,
-%        const long y_offset)
+%        const long y_offset,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
@@ -1117,13 +1123,16 @@ static void XShearImage(Image *image,const double degrees,
 %    o width, height, x_offset, y_offset: Defines a region of the image
 %      to shear.
 %
+%    o exception: Return any errors or warnings in this structure.
+%
 %
 */
-static void YShearImage(Image *image,const double degrees,
-                        const unsigned long width,const unsigned long height,long x_offset,
-                        const long y_offset)
+static MagickPassFail
+YShearImage(Image *image,const double degrees,
+	    const unsigned long width,const unsigned long height,long x_offset,
+	    const long y_offset,ExceptionInfo *exception)
 {
-#define YShearImageText  "[%s] Y Shear image..."
+#define YShearImageText  "[%s] Y Shear: %+g degrees, region %lux%lu%+ld%+ld...  "
 
   long
     y;
@@ -1201,7 +1210,7 @@ static void YShearImage(Image *image,const double degrees,
                 */
                 if (step > y_offset)
                   break;
-                p=GetImagePixelsEx(image,y+x_offset,0,1,image->rows,&image->exception);
+                p=GetImagePixelsEx(image,y+x_offset,0,1,image->rows,exception);
                 if (p == (PixelPacket *) NULL)
                   {
                     thread_status=MagickFail;
@@ -1220,7 +1229,7 @@ static void YShearImage(Image *image,const double degrees,
                 /*
                   Transfer pixels bottom-to-top.
                 */
-                p=GetImagePixelsEx(image,y+x_offset,0,1,image->rows,&image->exception);
+                p=GetImagePixelsEx(image,y+x_offset,0,1,image->rows,exception);
                 if (p == (PixelPacket *) NULL)
                   {
                     thread_status=MagickFail;
@@ -1235,17 +1244,19 @@ static void YShearImage(Image *image,const double degrees,
                 break;
               }
             }
-          if (!SyncImagePixelsEx(image,&image->exception))
+          if (!SyncImagePixelsEx(image,exception))
             thread_status=MagickFail;
 
 #if defined(HAVE_OPENMP)
-#  pragma omp critical
+#  pragma omp critical (GM_YShearImage)
 #endif
           {
             row_count++;
             if (QuantumTick(row_count,width))
-              if (!MagickMonitorFormatted(row_count,width,&image->exception,
-                                          YShearImageText,image->filename))
+              if (!MagickMonitorFormatted(row_count,width,exception,
+                                          YShearImageText,image->filename,
+					  degrees,width,height,
+					  x_offset,y_offset))
                 thread_status=MagickFail;
             
             if (thread_status == MagickFail)
@@ -1268,7 +1279,7 @@ static void YShearImage(Image *image,const double degrees,
             */
             if (step > y_offset)
               break;
-            p=GetImagePixelsEx(image,y+x_offset,0,1,image->rows,&image->exception);
+            p=GetImagePixelsEx(image,y+x_offset,0,1,image->rows,exception);
             if (p == (PixelPacket *) NULL)
               {
                 thread_status=MagickFail;
@@ -1299,7 +1310,7 @@ static void YShearImage(Image *image,const double degrees,
             /*
               Transfer pixels bottom-to-top.
             */
-            p=GetImagePixelsEx(image,y+x_offset,0,1,image->rows,&image->exception);
+            p=GetImagePixelsEx(image,y+x_offset,0,1,image->rows,exception);
             if (p == (PixelPacket *) NULL)
               {
                 thread_status=MagickFail;
@@ -1323,17 +1334,19 @@ static void YShearImage(Image *image,const double degrees,
             break;
           }
         }
-      if (!SyncImagePixelsEx(image,&image->exception))
+      if (!SyncImagePixelsEx(image,exception))
         thread_status=MagickFail;
 
 #if defined(HAVE_OPENMP)
-#  pragma omp critical
+#  pragma omp critical (GM_YShearImage)
 #endif
       {
         row_count++;
         if (QuantumTick(row_count,width))
-          if (!MagickMonitorFormatted(row_count,width,&image->exception,
-                                      YShearImageText,image->filename))
+          if (!MagickMonitorFormatted(row_count,width,exception,
+                                      YShearImageText,image->filename,
+				      degrees,width,height,
+				      x_offset,y_offset))
             thread_status=MagickFail;
         
         if (thread_status == MagickFail)
@@ -1342,6 +1355,8 @@ static void YShearImage(Image *image,const double degrees,
     }
   if (is_grayscale && IsGray(image->background_color))
     image->is_grayscale=True;
+
+  return status;
 }
 
 /*
@@ -1388,15 +1403,15 @@ static void YShearImage(Image *image,const double degrees,
 %
 %
 */
-MagickExport Image *RotateImage(const Image *image,const double degrees,
-  ExceptionInfo *exception)
+MagickExport Image *
+RotateImage(const Image *image,const double degrees,ExceptionInfo *exception)
 {
   double
     angle;
 
   Image
-    *integral_image,
-    *rotate_image;
+    *integral_image = (Image *) NULL,
+    *rotate_image = (Image *) NULL;
 
   long
     x_offset,
@@ -1432,8 +1447,8 @@ MagickExport Image *RotateImage(const Image *image,const double degrees,
   */
   integral_image=IntegralRotateImage(image,rotations,exception);
   if (integral_image == (Image *) NULL)
-    ThrowImageException3(ResourceLimitError,MemoryAllocationFailed,
-      UnableToRotateImage);
+    goto rotate_image_exception;
+
   shear.x=(-tan(DegreesToRadians(angle)/2.0));
   shear.y=sin(DegreesToRadians(angle));
   if ((shear.x == 0.0) || (shear.y == 0.0))
@@ -1459,24 +1474,43 @@ MagickExport Image *RotateImage(const Image *image,const double degrees,
   border_info.height=y_offset;
   rotate_image=BorderImage(integral_image,&border_info,exception);
   DestroyImage(integral_image);
+  integral_image=(Image *) NULL;
   if (rotate_image == (Image *) NULL)
-    ThrowImageException3(ResourceLimitError,MemoryAllocationFailed,
-      UnableToRotateImage);
+    goto rotate_image_exception;
+
   /*
     Rotate the image.
   */
   rotate_image->storage_class=DirectClass;
   rotate_image->matte|=rotate_image->background_color.opacity != OpaqueOpacity;
-  XShearImage(rotate_image,shear.x,width,height,x_offset,
-    (long) (rotate_image->rows-height)/2);
-  YShearImage(rotate_image,shear.y,y_width,height,
-    (long) (rotate_image->columns-y_width)/2,y_offset);
-  XShearImage(rotate_image,shear.x,y_width,rotate_image->rows,
-    (long) (rotate_image->columns-y_width)/2,0);
-  CropToFitImage(&rotate_image,shear.x,shear.y,width,height,True,exception);
+  
+  if (XShearImage(rotate_image,shear.x,width,height,x_offset,
+		  (long) (rotate_image->rows-height)/2,exception) != MagickPass)
+    goto rotate_image_exception;
+
+  if (YShearImage(rotate_image,shear.y,y_width,height,
+		  (long) (rotate_image->columns-y_width)/2,y_offset,exception)
+      != MagickPass)
+    goto rotate_image_exception;
+
+  if (XShearImage(rotate_image,shear.x,y_width,rotate_image->rows,
+		  (long) (rotate_image->columns-y_width)/2,0,exception)
+      != MagickPass)
+    goto rotate_image_exception;
+
+  if (CropToFitImage(&rotate_image,shear.x,shear.y,width,height,True,exception)
+      != MagickPass)
+    goto rotate_image_exception;
+
   rotate_image->page.width=0;
   rotate_image->page.height=0;
   return(rotate_image);
+
+ rotate_image_exception:
+
+  DestroyImage(integral_image);
+  DestroyImage(rotate_image);
+  return (Image *) NULL;
 }
 
 /*
@@ -1524,12 +1558,13 @@ MagickExport Image *RotateImage(const Image *image,const double degrees,
 %
 %
 */
-MagickExport Image *ShearImage(const Image *image,const double x_shear,
-  const double y_shear,ExceptionInfo *exception)
+MagickExport Image *
+ShearImage(const Image *image,const double x_shear,
+	   const double y_shear,ExceptionInfo *exception)
 {
   Image
-    *integral_image,
-    *shear_image;
+    *integral_image = (Image *) NULL,
+    *shear_image = (Image *) NULL;
 
   long
     x_offset,
@@ -1556,8 +1591,7 @@ MagickExport Image *ShearImage(const Image *image,const double x_shear,
   */
   integral_image=IntegralRotateImage(image,0,exception);
   if (integral_image == (Image *) NULL)
-    ThrowImageException3(ResourceLimitError,MemoryAllocationFailed,
-      UnableToShearImage);
+    goto shear_image_exception;
   shear.x=(-tan(DegreesToRadians(x_shear)/2.0));
   shear.y=sin(DegreesToRadians(y_shear));
   if ((shear.x == 0.0) || (shear.y == 0.0))
@@ -1576,22 +1610,38 @@ MagickExport Image *ShearImage(const Image *image,const double x_shear,
   border_info.width=x_offset;
   border_info.height=y_offset;
   shear_image=BorderImage(integral_image,&border_info,exception);
-  if (shear_image == (Image *) NULL)
-    ThrowImageException3(ResourceLimitError,MemoryAllocationFailed,
-      UnableToShearImage);
   DestroyImage(integral_image);
+  integral_image=(Image *) NULL;
+  if (shear_image == (Image *) NULL)
+    goto shear_image_exception;
   /*
     Shear the image.
   */
   shear_image->storage_class=DirectClass;
   shear_image->matte|=shear_image->background_color.opacity != OpaqueOpacity;
-  XShearImage(shear_image,shear.x,image->columns,image->rows,x_offset,
-    (long) (shear_image->rows-image->rows)/2);
-  YShearImage(shear_image,shear.y,y_width,image->rows,
-    (long) (shear_image->columns-y_width)/2,y_offset);
-  CropToFitImage(&shear_image,shear.x,shear.y,image->columns,image->rows,
-    False,exception);
+ 
+  if (XShearImage(shear_image,shear.x,image->columns,image->rows,x_offset,
+		  (long) (shear_image->rows-image->rows)/2,exception)
+      != MagickPass)
+    goto shear_image_exception;
+  
+  if (YShearImage(shear_image,shear.y,y_width,image->rows,
+		  (long) (shear_image->columns-y_width)/2,y_offset,exception)
+      != MagickPass)
+    goto shear_image_exception;
+  
+  if (CropToFitImage(&shear_image,shear.x,shear.y,image->columns,image->rows,
+		     False,exception) != MagickPass)
+    goto shear_image_exception;
+
   shear_image->page.width=0;
   shear_image->page.height=0;
+
   return(shear_image);
+
+ shear_image_exception:
+
+  DestroyImage(integral_image);
+  DestroyImage(shear_image);
+  return (Image *) NULL;
 }
